/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.layout;

import java.util.*;
import java.awt.*;

/**
 *  A node
 *
 * @author    William Grosso <grosso@smi.stanford.edu>
 */
public class IconGlyph implements Glyph {
    private String _shape;
    private boolean _filled;
    private Color _objectColor;
    private Color _textColor;
    private Rectangle _location;
    private Rectangle _oldLocation;
    private String _title;
    private ActorShape _shapeToDraw;
    private Object _noPointOfIntersection = new Object();
    private HashMap _startingPointsToEndingPointHashes;

    public IconGlyph(String shape, Color objectColor, Color textColor, boolean filled, String title) {
        setShape(shape);
        setObjectColor(objectColor);
        setTextColor(textColor);
        setFilled(filled);
        setTitle(title);
        _startingPointsToEndingPointHashes = new HashMap();
        _location = new Rectangle();
        _oldLocation = new Rectangle();
    }

    // The Copyable Interface
    public Object copy() {
        return new IconGlyph(_shape, _objectColor, _textColor, _filled, _title);
    }

    //  The Glyph Interface
    public void displayImage(ActorLocation location, Graphics g) {
        location.getBounds(_location);
        resetHashesIfNecessary();
        _shapeToDraw.drawShape(g, _objectColor, _filled, _location);
    }

    public void displayImage(Rectangle location, Graphics g) {
        _shapeToDraw.drawShape(g, _objectColor, _filled, location);
    }

    public void displaySelectedImage(ActorLocation location, Graphics g) {
        location.getBounds(_location);
        resetHashesIfNecessary();
        _shapeToDraw.drawHighlightedShape(g, _objectColor, Color.black, _filled, _location);
    }

    public void displaySelectedImage(Rectangle location, Graphics g) {
        _shapeToDraw.drawHighlightedShape(g, _objectColor, Color.black, _filled, location);
    }

    public Point getBoundaryPointForLine(Line line) {
        /*
         * Heavily relies on Point's hashcode doing the right thing (e.g.
         * that Point's hashcode only relies on the x and y coordinates and
         * not on any object-identity properties).
         */

        HashMap secondMap = (HashMap) _startingPointsToEndingPointHashes.get(line.startingPoint);
        if (null == secondMap) {
            secondMap = new HashMap();
            _startingPointsToEndingPointHashes.put(new Point(line.startingPoint), secondMap);
        }
        Object boundaryObject = secondMap.get(line.endingPoint);
        if (null == boundaryObject) {
            boundaryObject = _shapeToDraw.getBoundaryPointOnLine(line, _location);
            if (null == boundaryObject) {
                boundaryObject = _noPointOfIntersection;
            }
            secondMap.put(new Point(line.endingPoint), boundaryObject);
        }
        if (_noPointOfIntersection == boundaryObject) {
            return null;
        }
        return (Point) boundaryObject;
    }

    public boolean getFilled() {
        return _filled;
    }

    public Color getObjectColor() {
        return _objectColor;
    }

    public int getPreferredHeight() {
        return 48;
    }

    public int getPreferredWidth() {
        return 48;
    }

    public String getShape() {
        return _shape;
    }

    public Color getTextColor() {
        return _textColor;
    }

    private void resetHashesIfNecessary() {
        if (!_location.equals(_oldLocation)) {
            _startingPointsToEndingPointHashes.clear();
        }
        _oldLocation.x = _location.x;
        _oldLocation.y = _location.y;
        _oldLocation.height = _location.height;
        _oldLocation.width = _location.width;
    }

    public void setFilled(boolean filled) {
        _filled = filled;
    }

    public void setObjectColor(Color objectColor) {
        _objectColor = objectColor;
    }

    public void setShape(String shape) {
        _shape = shape;
        _shapeToDraw = ActorShapeFactory.getShapeForName(_shape);
        return;
    }

    public void setTextColor(Color textColor) {
        _textColor = textColor;
    }

    public void setTitle(String title) {
        _title = title;
    }
}
