/*
 * The contents of this file are subject to the Mozilla Public License
 * Version 1.1 (the "License");  you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License for
 * the specific language governing rights and limitations under the License.
 *
 * The Original Code is Protege-2000.
 *
 * The Initial Developer of the Original Code is Stanford University. Portions
 * created by Stanford University are Copyright (C) 2001.  All Rights Reserved.
 *
 * Protege-2000 was developed by Stanford Medical Informatics
 * (http://www.smi.stanford.edu) at the Stanford University School of Medicine
 * with support from the National Library of Medicine, the National Science
 * Foundation, and the Defense Advanced Research Projects Agency.  Current
 * information about Protege can be obtained at http://protege.stanford.edu
 *
 * Contributor(s):
 */

package edu.stanford.smi.protegex.widget.oil;

import java.util.*;
import java.awt.*;
import javax.swing.*;
import edu.stanford.smi.protege.action.*;
import edu.stanford.smi.protege.event.*;
import edu.stanford.smi.protege.model.*;
import edu.stanford.smi.protege.util.*;
import edu.stanford.smi.protege.ui.*;
import edu.stanford.smi.protege.widget.*;

/** This is a modified InstanceListWidget for showing expressions. */
public class ExpressionListWidget extends ModifiableAbstractListWidget {

    private AllowableAction createInstanceAction;
    private AllowableAction addInstancesAction;
    private AllowableAction itsRemoveInstancesAction;
    private AllowableAction itsDeleteInstancesAction;

    private ExpressionWidgetHelper itsExpressionWidgetHelper;


    public ExpressionListWidget() {
      super();
      itsExpressionWidgetHelper = new ExpressionWidgetHelper(this);
    }

    private FrameListener itsInstanceListener =
        new FrameAdapter() {
            public void browserTextChanged(FrameEvent event) {
                super.browserTextChanged(event);
                // Log.trace("changed", this, "browserTextChanged");
                repaint();
            }
        }
    ;

    public static boolean isSuitable(Cls cls, Slot slot, Facet facet) {
      return ExpressionWidgetHelper.isSuitable(cls, slot, facet, true);
      // true -> multiple value mode
    }

    public void addItem(Object item) {
        super.addItem(item);
        addListener(CollectionUtilities.createCollection(item));
	itsExpressionWidgetHelper.prettyPrintExpressions();
    }

    public void addItems(Collection items) {
        super.addItems(items);
        addListener(items);
	itsExpressionWidgetHelper.prettyPrintExpressions();
    }

    private void addListener(Collection values) {
        Iterator i = values.iterator();
        while (i.hasNext()) {
            Instance instance = (Instance) i.next();
            instance.addFrameListener(itsInstanceListener);
        }
    }

    public void dispose() {
        super.dispose();
        removeListener(getValues());
	itsExpressionWidgetHelper.removeClassesAndAllInstancesListeners();
    }

    private Action getAddInstancesAction() {
        addInstancesAction = new AddAction("Select Instances") {
            public void onAdd() {
                Collection clses = getCls().getTemplateSlotAllowedClses(getSlot());
                addItems(DisplayUtilities.pickInstances(ExpressionListWidget.this, clses, (String) getValue(SHORT_DESCRIPTION)));
            }
        };
        return addInstancesAction;
    }

    public Action getCreateInstanceAction() {
        createInstanceAction = new CreateAction("Create Instance") {
            public void onCreate() {
                Collection clses = getCls().getTemplateSlotAllowedClses(getSlot());
                Cls cls = DisplayUtilities.pickConcreteCls(ExpressionListWidget.this, clses);
                if (cls != null) {
                    Instance instance = getKnowledgeBase().createInstance(null, cls);
                    if (instance instanceof Cls) {
                        ((Cls) instance).addDirectSuperclass(getKnowledgeBase().getRootCls());
                    }
                    showInstance(instance);
                    addItem(instance);
                }
            }
        };
        return createInstanceAction;
    }

    private Action getDeleteInstancesAction() {
        itsDeleteInstancesAction = new DeleteInstancesAction("Delete Selected Instances", this);
        return itsDeleteInstancesAction;
    }

    private Action getRemoveInstancesAction() {
        itsRemoveInstancesAction = new RemoveAction("Remove Selected Instances", this) {
            public void onRemove(Object o) {
                removeItem(o);
	        itsExpressionWidgetHelper.prettyPrintExpressions();
            }
        };
        return itsRemoveInstancesAction;
    }

    private Action getViewInstanceAction() {
        return new ViewAction("View Selected Instances", this) {
            public void onView(Object o) {
                showInstance((Instance) o);
            }
        };
    }

    public void initialize() {
        Action viewAction = getViewInstanceAction();
        super.initialize(viewAction);
        addButton(viewAction);
        addButton(getCreateInstanceAction());
        addButton(new ReferencersAction(this), false);
        addButton(getAddInstancesAction());
        addButton(getRemoveInstancesAction());
        addButton(getDeleteInstancesAction(), false);
        setRenderer(new FrameRenderer());
	// --- added:
	itsExpressionWidgetHelper.initialize();
	JPanel panel = getBorderLayoutPanel();
	JTextArea expressionArea = new JTextArea(6, 20);
	expressionArea.setForeground(Color.blue);
	expressionArea.setEditable(false);
	LabeledComponent expressionComponent = 
	  new LabeledComponent("Expressions", 
	    new JScrollPane(expressionArea));
	expressionComponent.setBorder(
	  BorderFactory.createEmptyBorder(0,5,5,5)); // t l b r insets
	panel.add(expressionComponent, BorderLayout.CENTER);
	itsExpressionWidgetHelper.setExpressionTextArea(expressionArea);
	itsExpressionWidgetHelper.addClassListeners();
	itsExpressionWidgetHelper.addAllInstancesListeners();
    }

    private void removeListener(Collection values) {
        Iterator i = values.iterator();
        while (i.hasNext()) {
            Instance instance = (Instance) i.next();
            instance.removeFrameListener(itsInstanceListener);
        }
    }

    public void setEditable(boolean b) {
        createInstanceAction.setAllowed(b);
        addInstancesAction.setAllowed(b);
        itsRemoveInstancesAction.setAllowed(b);
        itsDeleteInstancesAction.setAllowed(b);
    }

    public void setValues(Collection values) {
        removeListener(getValues());
        addListener(values);
        super.setValues(values);
	itsExpressionWidgetHelper.prettyPrintExpressions();
    }
}


