package fi.helsinki.cs.keltsi.jenardf;

import com.hp.hpl.jena.daml.*;
import java.util.Iterator;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;
import java.util.LinkedList;
import com.hp.hpl.jena.vocabulary.DAML_OIL;

/**
 *  Class representing RDFS.Class -objects
 *  There are three distinct subclasses, 
 *  (1) a KeltsiClass of Services
 *  (2) a KeltsiClass of Products 
 *  (3) a KeltsiClass of an additional ontology.
 *  Since they behave identically,
 *  subclassing has been removed.
 *  Methods to make the distinction are provided.
 */
public class KeltsiClass extends KeltsiResource {
    private KeltsiClass top;

    /* 
     *  Constructor not to be called directly, use
     *  KeltsiDAML.getKelstiResource instead.
     */
    KeltsiClass(DAMLClass cls) {
        super(cls);
    }

    /*
     *  Iterator over the subclasses of this class.
     *  The iterator holds either only the direct
     *  subclasses (close is false) or the transitive
     *  closure of the inverse of the property subClassOf.
     *  @param close Supply transitive closure if true.
     *  @return Iterator over subclasses of this class.
     */
    public Iterator getSubClasses(boolean close) {
        return classList(getDAMLClass().getSubClasses(close));
    }

    /*
     *  Iterator over the superclasses of this class.
     *  The iterator holds either only the direct
     *  superclasses (close is false) or the transitive
     *  closure of the property subClassOf.
     *  @param close Supply transitive closure if true.
     *  @return Iterator over superclasses of this class.
     */
    public Iterator getSuperClasses(boolean close) {
        return classList(getDAMLClass().getSuperClasses(close));
    }

    /*
     *  Is the supplied class a subclass of this class?
     *  @param cls The candidate to be a subclass.
     *  @return Boolean indicating wether the class is a
     *  subclass of this class.
     */
    public boolean hasSubClass(KeltsiClass cls) {
        return getDAMLClass().hasSubClass(cls.getDAMLClass());
    }

    /*
     *  Is the supplied class a superclass of this class?
     *  @param cls The candidate to be a superclass.
     *  @return Boolean indicating wether the class is a
     *  superclass of this class.
     */
    public boolean hasSuperClass(KeltsiClass cls) {
        /* Doesn't work without try-catch! Propably bumps into a 
           non-DAMLClass resource.
           Could also return cls.hasSubClass(this);
        */
        try {
            return getDAMLClass().hasSuperClass(cls.getDAMLClass());
        } catch(Exception ex) {
            return false;
        }
    }
    
    /*
     *  Get the direct superclass of this class.
     *  Assuming there is only one, as is the case in the
     *  current data.
     *  @return The direct superclass of this class.
     */
    public KeltsiClass getSuper() {
        try {
            DAMLClass jenaSuper = (DAMLClass)
                    getDAMLClass().getPropertyValue(RDFS.subClassOf);
            if (jenaSuper.equals(DAML_OIL.Thing)) {
                return null;
            }
            return (KeltsiClass) getModel().getResource(jenaSuper.getURI());
        } catch(Exception ex) {
            return null;
        }
    }

    /*
     *  Get the topmost superclass of this class
     *  before rdfs:Resource or daml:Thing or whatever.
     *  @return The topmost superclass of this class.
     */
    public KeltsiClass getTop() {
        if (top == null) {
            KeltsiClass cls = this;
            KeltsiClass superC = getSuper();
            while (superC != null) {
                cls = superC;
                superC = cls.getSuper();
            }
            top = cls;
        }
        return top;
    }

    /**
     *  Provide an iterator over all the possible properties
     *  that are immediate subproperties of property.
     *  @param property The superproperty whose subproperties
     *                  can only be returned
     *  @result An iterator.
     */
    public Iterator getDefinedSubPropertiesOf(DAMLProperty property) {
        LinkedList properties = new LinkedList();
        Iterator props = getDAMLClass().getDefinedProperties();
        while (props.hasNext()) {
            DAMLProperty subProperty = (DAMLProperty)props.next();
            PropertyAccessor pa = subProperty.prop_subPropertyOf();
            if (pa.hasValue(property)) {
                properties.add(subProperty);
            }
        }
        return properties.iterator();
    }

    /* 
     *  Get the count of instances of this class.
     *  If close is false, only the direct instances will be
     *  included. If close is true, instances will be gathered
     *  from the transitive closure of the inverse of
     *  rdfs:subPropertyOf and this class.
     *  @param close Supply transitive closure if true.
     *  @return The count of instances.
     */
    public int getInstanceCount(boolean close) {
        int count = 0;
        Iterator damlInstances = getDAMLInstances(close);
        while(damlInstances.hasNext()) {
            ++count;
            damlInstances.next();
        }
        return count;
    }

    /* 
     *  Get an iterator over instances of this class.
     *  If close is false, only the direct instances will be
     *  included. If close is true, instances will be gathered
     *  from the transitive closure of the inverse of
     *  rdfs:subPropertyOf and this class.
     *  @param close Supply transitive closure if true.
     *  @return An iterator over instances.
     */
    public Iterator getInstances(boolean close){
        Iterator damlInstances = getDAMLInstances(close);
        LinkedList instances = new LinkedList();
        while(damlInstances.hasNext()) {
            instances.add(getModel().getKeltsiResource(
                    (DAMLCommon) damlInstances.next()));
        }
        return instances.iterator();
    }

    /* 
     * Does this class have Services as instances?
     * In other words, is this class a subclass of ServiceRoot.
     * If a class is neither a ServiceClass nor a ProductClass
     * it is an auxiliary OntologyClass (Location etc).
     * @return True if this class has Services as instances.
     */
    public boolean isServiceClass() {
        return (getTop().equals(Keltsi.ServiceRoot));
    }
    
    /* 
     * Does this class have Products as instances?
     * In other words, is this class a subclass of ProductRoot.
     * At the moment there are no Products as instances,
     * but that need not to be the case.
     * If a class is neither a ServiceClass nor a ProductClass
     * it is an auxiliary OntologyClass (Location etc).
     * @return True if this class has Products as instances.
     */
    public boolean isProductClass() {
        return (getTop().equals(Keltsi.ProductRoot));
    }

    /* 
     * Is this class right below Thing or Resource or whatever?
     * @return True if this class is topmost in it's ontology.
     */
    public boolean isTop() {
        return (getTop().equals(this));
    }

    protected Iterator getDAMLInstances(boolean close) {
        if (close) {
            return getDAMLClass().getInstances();
        } else {
            return (Iterator) getModel().select(null, RDF.type, 
                                             (RDFNode) resource);
        }
    }

    private DAMLClass getDAMLClass() {
        return (DAMLClass)resource;
    }

    private Iterator classList(Iterator damlClasses) {
        LinkedList kClasses = new LinkedList();
        while (damlClasses.hasNext()) {
            Resource damlClass = null;
            damlClass = (Resource) damlClasses.next();
            if(damlClass != null && damlClass instanceof DAMLClass 
                    && !damlClass.equals(DAML_OIL.Thing)) {
                KeltsiClass kClass =(KeltsiClass) 
                            getModel().getKeltsiResource((DAMLClass)damlClass);
                kClasses.addFirst(kClass);
            }
        }
        return kClasses.iterator();
    }

}
