package fi.helsinki.cs.keltsi.jenardf;

import com.hp.hpl.jena.daml.common.DAMLModelImpl;
import com.hp.hpl.jena.daml.common.DAMLSelector;
import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.jena.vocabulary.DAML_OIL;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;

import com.hp.hpl.mesa.rdf.jena.common.*;
import com.hp.hpl.jena.rdf.query.*;

import fi.helsinki.cs.keltsi.util.*;

import java.util.*;
import java.io.*;

/**
 *  The model extends DAMLModelImpl for now. Better solution
 *  could be to throw all DAML-stuff away and use basic ModelMem
 *  or ModelRDB. DAMLModel seems to be very slow in some aspects.
 *  
 */
public class KeltsiDAML extends DAMLModelImpl {

    public static final String keltsiNS =
                "http://protege.stanford.edu/Keltsi#";

    public static final String RDFSNS = DAML_OIL.label.getNameSpace();
    public static final String RDFNS = DAML_OIL.type.getNameSpace();

    public static final int SUBJECT = 1;
    public static final int PREDICATE = 2;
    public static final int OBJECT = 4;
    public String error;
    private HashMap inverse;
    private HashMap ontology;

    public String modelName = "KeltsiData";

    /******* Methods to access the schema and instance models ********/

    private String baseURL = "http://www.cs.helsinki.fi/group/keltsi/";
    public void setBaseURL(String baseURL) {
        this.baseURL = baseURL;
    }
    public String getBaseURL() {
        return baseURL;
    }

    public void addFile(String fileName) {
        try {
            read(baseURL + fileName);
        } catch (Exception ex) {
            try {
                // Use the class loader to find the input file.
                InputStream in = fi.helsinki.cs.keltsi.jenardf.KeltsiDAML.class
                                .getClassLoader()
                                .getResourceAsStream(fileName);
                if (in == null) {
                    throw new IllegalArgumentException(
                            "File: " + fileName + " not found");
                }
                read(new InputStreamReader(in), "");
            } catch (Exception ex2) {
                System.err.println("Exception: "+ex2);
                ex2.printStackTrace(System.err);
                error = ex2.toString();
            }
        }
    }

    /**
     * load model from file
     */
    public void loadModel(String modelName) {
        this.modelName = modelName;
        addFile("restaurant.rdfs");
        addFile("Location.rdfs");
        addFile(modelName + ".rdfs");
        addFile(modelName + ".rdf");
        Keltsi.initialize(this);
        initOntologies();
    }

    private void initOntologies() {
        Keltsi.ServiceRoot = (KeltsiClass) getKeltsiResource(Keltsi.Service);
        Keltsi.ProductRoot = (KeltsiClass) getKeltsiResource(Keltsi.Product);

        inverse = new HashMap();
        inverse.put(keltsiNS + "franchisee",
                    getDAMLValue(keltsiNS + "franchiseeOf"));
        ontology = new HashMap();

        String[] pverbs = {"retails", "rents", "maintains", "wholesales",
                           "manufactures "};
        ontology.put("productclass",
                new Ontology(Keltsi.ProductRoot, pverbs, "productclass",
                             Keltsi.ServiceRoot));

        String[] sverbs = {"franchises"};
        ontology.put("serviceclass",
                new Ontology(Keltsi.ServiceRoot, sverbs, "serviceclass",
                             Keltsi.ServiceRoot));

        String[] rverbs = {"restaurantType"};
        KeltsiClass restaurantType = (KeltsiClass) getKeltsiResource(
                "http://protege.stanford.edu/restaurant#RestaurantType");
        KeltsiClass restaurants = (KeltsiClass) getKeltsiResource(
                Keltsi.getURI() + "RestaurantsAndCatering");
        ontology.put("restauranttype",
                new Ontology(restaurantType, rverbs, "restauranttype",
                             restaurants));

        String[] averbs = {"ambiance"};
        KeltsiClass ambiance = (KeltsiClass) getKeltsiResource(
                "http://protege.stanford.edu/restaurant#Ambiance");
        ontology.put("ambiance",
                new Ontology(ambiance, averbs, "ambiance",
                             restaurants));

        String[] rtverbs = {"rating"};
        KeltsiClass rating = (KeltsiClass) getKeltsiResource(
                "http://protege.stanford.edu/restaurant#Rating");
        ontology.put("rating",
                new Ontology(rating, rtverbs, "rating",
                             restaurants));

        String[] lverbs = {"location"};
        KeltsiClass location = (KeltsiClass) getKeltsiResource(
                "http://protege.stanford.edu/Location#Area");
        ontology.put("location",
                new Ontology(location, lverbs, "location",
                             Keltsi.ServiceRoot));
    }

    public Iterator getOntologies () {
        return ontology.values().iterator();
    }

    public String[] getVerbs(String ontologyURI) {
        return (String[]) ontology.get(ontologyURI);
    }

    public DAMLProperty getInverseProperty(String propertyURI) {
        return (DAMLProperty) inverse.get(propertyURI);
    }

    /* Methods to keep account of resources */


    public KeltsiResource getKeltsiResource(String uri) {
        if (uri == null) {
            return null;
        }
        DAMLCommon jenaResource = getDAMLValue(uri);
        KeltsiResource keltsiResource = null;
        if (jenaResource instanceof DAMLInstance) {
            keltsiResource = new Service((DAMLInstance) jenaResource);
        } else if (jenaResource instanceof DAMLClass) {
            keltsiResource = new KeltsiClass((DAMLClass) jenaResource);
        }
        return keltsiResource;
    }

    public KeltsiResource getKeltsiResource(DAMLCommon jenaResource) {
        return getKeltsiResource(jenaResource.getURI());
    }

    /******* Methods to query the models ********/

    /**
     * execute RDQL-query
     */
    public QueryResults executeQuery(String queryString) {
        Query query = new Query(queryString) ;
        query.setSource(this);
        QueryExecution qe = new QueryEngine(query) ;
        return qe.exec();
    }

    /**
     * Execute RDQL-query beginning "SELECT ?x WHERE"
     */
    public Iterator xQuery(String queryString) {
        HashSet set = new HashSet();
        xQuerySet(queryString, set);
        return set.iterator();
    }

    /**
     * Execute RDQL-query beginning "SELECT ?x WHERE"
     * and add results to set. Makes possible to
     * create union of several queryresults.
     */
    public void xQuerySet(String queryString, HashSet set) {
        QueryResults results = executeQuery(queryString);
        while(results.hasNext()) {
            ResultBinding resultBinding = (ResultBinding)results.next();
            set.add(resultBinding.get("x"));
        }
    }
        
    /**
     * Query with a selector
     */
    public ResultIterator select(Resource subject,
                                 Property predicate,
                                 RDFNode object) {
        int select = 0;
        if (subject == null) {
            select += SUBJECT;
        }
        if (predicate == null) {
            select += PREDICATE;
        }
        if (object == null) {
            select += OBJECT;
        }
        DAMLSelector damlSelector =
                new DAMLSelector(subject, predicate, object);
        try {
            StmtIterator si =
                    listStatements(damlSelector);
            return new ResultIterator(si, select);
        } catch (Exception ex) {
            return null;
        }
    }

    /**
     * Contruct query-node
     */
    public String qnode(String node) {
        if (node == null) {
            node = "?x";
        } else {
            node = "<" + node + ">";
        }
        return node;
    }

    /**
     * Contruct query-triple
     */
    public String triple(String sURI, String pURI, String oURI) {
        return "(" + qnode(sURI) + ", " + qnode(pURI) + ", " + qnode(oURI) + ") ";
    }
    
    private String mapQuery;
    public String getPreviousMapQueryString(){
        return mapQuery;
    }

    private String getNextName(Iterator names) {
        while(names.hasNext()) {
            String name = (String) names.next();
            if (ontology.containsKey(name)){
                return name;
            }
        }
        return null;
    }

    private boolean hasSProperty(DAMLCommon subject, Property property, DAMLClass value) {
        try {
            if (subject.hasProperty(property, value)) {
                return true;
            }
        } catch (Exception ex) {}

        Iterator subClasses = value.getSubClasses(true);
        while (subClasses.hasNext()) {
            value = (DAMLClass) subClasses.next();
            try {
                if (subject.hasProperty(property, value)) {
                    return true;
                }
            } catch (Exception ex) {}
        }
        return false;
    }

    public Iterator mapQuery(HashMap map, boolean close) {
        String sclassURI = (String) map.get("serviceclassuri");
        DAMLClass serviceClass = (DAMLClass)getDAMLValue(sclassURI);
        
        /* TODO: Optimization, here is the place to do it!
           Could choose any (property, SValue) pair to begin with, 
           smallest is cheapest. But which one is smallest?
        */
        HashMap query = new HashMap();
        Iterator names = map.keySet().iterator();
        while(names.hasNext()) {
            String name = (String) names.next();
            if (ontology.containsKey(name)){
                Ontology ont = (Ontology) ontology.get(name);
                String value = (String) map.get(name);
                String predicate = (String) map.get(name + "verb");
                if (predicate == null) {
                    predicate = ont.defaultVerb();
                }
                try {
                    query.put(getProperty(keltsiNS+predicate), 
                              getDAMLValue(value));
                } catch(Exception ex) {}
            }
        }
        HashSet set = new HashSet();
        /* So now we start with the pair (RDF.type, serviceclass)
           because its easiest.
        */
        Iterator services = serviceClass.getInstances();

        while (services.hasNext()) {
            DAMLInstance service = (DAMLInstance) services.next();
            Iterator properties = query.keySet().iterator();
            boolean include = true;
            while (properties.hasNext()) {
                Property property = (Property)properties.next();
                DAMLClass value = (DAMLClass)query.get(property);
                if (!hasSProperty(service, property, value)) {
                    include = false;
                    break;
                }
            }
            if (include) {
               // Could add instances of Service like this
               // set.add(Service.getService(service));
               // But to be compatible with api, add DAMLInstance
               set.add(service);
            }
        }
        return set.iterator();
    }

    public String getSynonymClassURI(String synonym) {
        Iterator iterator = select(null,
                                   Keltsi.synonym,
                                   new LiteralImpl(synonym));
        if (iterator.hasNext()) {
            return ((DAMLCommon) iterator.next()).getURI();
        } else {
            // search for ServiceClasses with this label
            return null;
        }
    }
    
    public String inverseName(String localName) {
        if (localName == null) {
            return null;
        }
        if (localName.endsWith("Of")){
            return localName.substring(0, localName.length() - 2);
        } else {
            return localName + "Of";
        }
    }
    
    public static void main(String args[])
    throws RDFException {
        KeltsiDAML model = new KeltsiDAML();
        model.setBaseURL("http://192.168.1.1/Keltsi/");
        model.loadModel(model.modelName);
    }
}
