package fi.helsinki.cs.keltsi.jenardf;

import fi.helsinki.cs.keltsi.util.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;

import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.jena.daml.*;
import java.util.LinkedList;
import java.util.Iterator;

/*
 *  Class to encapsulate a Service. 
 *  Provides access to it's properties including
 *  advertisement and contacts.
 */
public class Service extends KeltsiResource {

    /* 
     * Constructor called only from KeltsiDAML.
     * Use KelstiDAML.getKeltsiResource to access Services.
     */
    Service(DAMLInstance service) {
        super(service);
    }

    /* 
     *  Get the class (rdf:type) of the Service.
     *  @return The KeltsiClass object that corresponds to the Service's type.
     */
    public KeltsiClass getServiceClass() {
        return (KeltsiClass) getModel().getKeltsiResource((DAMLClass) getType());
    }

    /*
     * The label of the Service. First tries to retrieve Keltsi.label,
     * then the advertisements label. Best practice would be rdfs:label
     * but due to Protege this was not possible.
     * TODO: Language versions.
     * @return The label of the service as a String.
     */
    public String getLabel() {
        String label = getSingleLiteralValue(Keltsi.label);
        if (label == null) {
            Advertisement adv = getAdvertisement();
            if (adv != null) {
                label = adv.getLabel();
            }
        }
        return label;
    }

    /* 
     * A short description of the Service. Perhaps better would be to
     * to use rdfs:comment, but that is not available for instances
     * in Protege.
     * TODO: Language versions.
     * @return The description of the Service.
     */
    public String getDescription() {
        return getSingleLiteralValue(Keltsi.description);
    }

    /*
     * Creates an iterator holding values of the supplied property,
     * (<this>, <property>, ?service).
     * Fails (and returns an empty or partial list) if a 
     * non-Service value is encountered.
     * @return Iterator of property values of type Service.
     * @param property The property whose values are to be iterated.
     */
    public Iterator getServiceValues(DAMLProperty property) {
        NodeIterator ni = resource.getPropertyValues(property);
        LinkedList list = new LinkedList();
        try {
            while (ni.hasNext()) {
                Service service = (Service) getModel()
                        .getKeltsiResource((DAMLInstance)ni.next());
                if (service != null) {
                    list.add(service);
                }
            }
        } catch(Exception ex) {}
        return list.iterator();
    }

    /*
     * Creates an iterator holding the Service-values of the supplied 
     * properties inverse, (?service, <property>, <this>). 
     * Fails (and returns an empty or partial list) if a 
     * non-Service value is encountered.
     * @return Iterator over inverse properties values of type Service.
     * @param property The property whose inverses values are to be iterated.
     */
    public Iterator getServiceInverseValues(DAMLProperty property) {
        ResultIterator ri = getModel().select(null, property, getResource());
        LinkedList list = new LinkedList();
        try {
            while (ri.hasNext()) {
                Service service = (Service) getModel()
                        .getKeltsiResource((DAMLInstance)ri.next());
                if (service != null) {
                    list.add(service);
                }
            }
        } catch(Exception ex) {ex.printStackTrace();}
        return list.iterator();
    }

    /*
     * Creates an iterator holding KeltsiClass-values of the supplied 
     * property, (<this>, <property>, ?class).
     * Fails (and returns an empty or partial list) if a 
     * non-KeltsiClass value is encountered.
     * @return Iterator of property values of type KeltsiClass.
     * @param property The property whose values are to be iterated.
     */
    public Iterator getClassValues(DAMLProperty property) {
        NodeIterator ni = resource.getPropertyValues(property);
        LinkedList list = new LinkedList();
        try {
            while (ni.hasNext()) {
                KeltsiClass cls = (KeltsiClass) getModel()
                        .getKeltsiResource((DAMLClass) ni.next());
                if (cls != null) {
                    list.add(cls);
                }
            }
        } catch(Exception ex) {ex.printStackTrace();}
        return list.iterator();
    }

    /*
     * Creates an iterator holding Literal values of the supplied 
     * property, (<this>, <property>, ?literal).
     * Fails (and returns an empty or partial list) if a 
     * non-Literal value is encountered.
     * Literals are transformed to Strings.
     * This is not good practice, if the xml:lang attribute
     * is of use.
     * @return Iterator of property values of type Literal,
     * represented as Strings in the list.
     * @param property The property whose values are to be iterated.
     */
    public Iterator getLiteralValues(DAMLProperty property) {
        NodeIterator ni = resource.getPropertyValues(property);
        LinkedList list = new LinkedList();
        try {
            while (ni.hasNext()) {
                try {
                    Literal literal = (Literal)ni.next();
                    list.add(literal.getString());
                } catch(Exception ex) {ex.printStackTrace();}
            }
        } catch(Exception ex2) {ex2.printStackTrace();}
        return list.iterator();
    }

    /*
     * Get the advertisement. There should be exactly one.
     * @return The advertisement of this Service.
     */
    public Advertisement getAdvertisement() {
        Advertisement advertisement = null;
        ResultIterator resIter = getModel().select(
                null, Keltsi.services, (DAMLInstance) resource);
        if (resIter.hasNext()) {
            try {
                advertisement = new Advertisement(
                        (DAMLInstance)resIter.next());
            } catch (Exception ex) {}
        }
        resIter.close();
        return advertisement;
    }

    /*
     * Get an iterator for contacts of this Service.
     * There should be one or more.
     * @return An iterator over the contacts of this service.
     */
    public Iterator getContacts() {
        LinkedList contacts = new LinkedList();
        NodeIterator iter = ((DAMLInstance) resource)
                .getPropertyValues(Keltsi.contact);
        try {
            while (iter.hasNext()) {
                contacts.add(new Contact((DAMLInstance)iter.next()));
            }
        } catch (Exception ex) {
            try {iter.close();} catch(Exception exx) {}
        }
        return contacts.iterator();
    }

}

