/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package
 * Created            4 Jan 2002
 * Filename           $RCSfile: DAMLListModel.java,v $
 * Revision           $Revision: 1.1 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2002/01/10 14:18:51 $
 *               by   $Author: ijd $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package examples.daml.ex1;


// Imports
///////////////
import javax.swing.*;
import java.util.*;
import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;



/**
 * Swing list model that draws its data from the RDF model and a filter
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian_Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: DAMLListModel.java,v 1.1 2002/01/10 14:18:51 ijd Exp $
 */
public class DAMLListModel
    extends AbstractListModel
{
    // Constants
    //////////////////////////////////


    // Static variables
    //////////////////////////////////


    // Instance variables
    //////////////////////////////////

    /** Reference to the application object */
    private DAMLViewer m_viewer = null;

    /* The underling DAMLModel for this list model */
    private DAMLModel m_model = null;

    /** The filter that will select which DAML objects to show */
    private DAMLFilter m_filter = null;

    /** If true, sort the output alphbetically */
    private boolean m_sort = true;

    /** The elements of my list */
    private ArrayList m_listEntries = new ArrayList();



    // Constructors
    //////////////////////////////////

    /**
     * Construct the list model from a given DAML model and a filter
     * that will select a subset of the DAML values for rendering in
     * the list
     *
     * @param viewer Application object that holds global preferences
     * @param model The DAML model that contains the data
     * @param filter A DAMLFilter that will select certain DAML objects
     * @param sort If true, the rendered objects will be sorted alphabetically
     */
    public DAMLListModel( DAMLViewer viewer, DAMLModel model, DAMLFilter filter, boolean sort ) {
        m_model = model;
        m_filter = filter;
        m_sort = sort;
        m_viewer = viewer;

        reload( false );
    }


    // External signature methods
    //////////////////////////////////

    /**
     * Answer the size of the list
     *
     * @return The size as an integer
     */
    public int getSize() {
        return m_listEntries.size();
    }


    /**
     * Answer the element of the list at the given index
     *
     * @param index The index into the list
     * @return A list element
     */
    public Object getElementAt( int index ) {
        return m_listEntries.get( index );
    }


    /**
     * Reload the model, by re-applying the filter to the underlying
     * model.
     */
    public void reload( boolean notify ) {
        // get the filtered objects
        m_listEntries.clear();

        for (Iterator i = m_filter.filterDAMLValues( m_model );  i.hasNext();  ) {
            Resource r = (Resource) i.next();

            // only show anon values if the user says OK
            if (!r.isAnon() ||  m_viewer.getShowAnonymous()) {
                m_listEntries.add( r );
            }
        }

        // sort if necessary
        if (m_sort) {
            Collections.sort( m_listEntries, new DAMLDisplayComparator() );
        }

        // notify my listeners
        if (notify) {
            fireIntervalAdded( this, 0, m_listEntries.size() - 1 );
        }
    }



    // Internal implementation methods
    //////////////////////////////////


    //==============================================================================
    // Inner class definitions
    //==============================================================================


}
