/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package
 * Created            6 Jan 2002
 * Filename           $RCSfile: DAMLTableModel.java,v $
 * Revision           $Revision: 1.1 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2002/01/10 14:18:51 $
 *               by   $Author: ijd $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package examples.daml.ex1;


// Imports
///////////////
import javax.swing.*;
import javax.swing.table.*;
import com.hp.hpl.jena.daml.*;
import java.util.*;
import com.hp.hpl.mesa.rdf.jena.model.*;



/**
 * Provides the data model for a JTable displaying a 2-column table of properties
 * and values for a given object.
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian_Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: DAMLTableModel.java,v 1.1 2002/01/10 14:18:51 ijd Exp $
 */
public class DAMLTableModel
    extends AbstractTableModel
{
    // Constants
    //////////////////////////////////


    // Static variables
    //////////////////////////////////


    // Instance variables
    //////////////////////////////////

    /** The DAML model that this table is providing a view of */
    private DAMLModel m_model = null;

    /** The filter to select the statements from the model */
    private DAMLStatementFilter m_filter = null;

    /** The cached data from the model */
    private ArrayList m_statements = new ArrayList();

    /** For comparing the names of wro properties */
    private DAMLDisplayComparator m_comparator = new DAMLDisplayComparator();


    // Constructors
    //////////////////////////////////

    /**
     * Construct a table model that uses a DAML filter to select particular
     * values from the model to display. In this case, we assume the filtered
     * values to be statements, and the table will contain two columns: the
     * property, and the value as an RDFNode
     */
    public DAMLTableModel( DAMLModel m, DAMLStatementFilter filter ) {
        m_model = m;
        m_filter = filter;

        reload( false );
    }


    // External signature methods
    //////////////////////////////////

    /**
     * Reload the cached table data from the model.  If notify is true, also
     * notify the table listeners that the underlying data has changed.
     */
    public void reload( boolean notify ) {
        try {
            m_statements.clear();

            // cache the filtered statements
            for (StmtIterator i = m_filter.filterDAMLStatements( m_model );  i.hasNext();  ) {
                m_statements.add( i.next() );
            }

            // sort by the name of the predicate
            Collections.sort( m_statements,
                              new Comparator() {
                                  public int compare( Object o1, Object o2 ) {
                                      // get the predicates from the statements (i.e. sort statements in order of predicate names)
                                      return m_comparator.compare( ((Statement) o1).getPredicate(), ((Statement) o2).getPredicate() );
                                  }
                              } );

            if (notify) {
                fireTableDataChanged();
            }
        }
        catch (RDFException e) {
            // TODO can handle this more gracefully
            System.err.println( "RDF Exception: " + e );
            e.printStackTrace();
        }
    }


    /**
     * Answer the number of rows in the table
     */
    public int getRowCount() {
        return m_statements.size();
    }


    /**
     * Answer that there are two columns in the table
     */
    public int getColumnCount() {
        return 2;
    }


    /**
     * Answer the object at the given cell position
     */
    public Object getValueAt( int row, int col ) {
        if (col == 0) {
            return ((Statement) m_statements.get( row )).getPredicate();
        }
        else {
            return ((Statement) m_statements.get( row )).getObject();
        }
    }


    /**
     * Answer column names
     */
    public String getColumnName( int i ) {
        return (i == 0) ? "Property" : "Value";
    }


    /**
     * Answer column class
     */
    public Class getColumnClass( int col ) {
        return Resource.class;
    }



    // Internal implementation methods
    //////////////////////////////////



    //==============================================================================
    // Inner class definitions
    //==============================================================================

}


