/*****************************************************************************
 * Source code information
 * -----------------------
 * Original author    Ian Dickinson, HP Labs Bristol
 * Author email       Ian_Dickinson@hp.com
 * Package
 * Created            4 Jan 2002
 * Filename           $RCSfile: DAMLViewer.java,v $
 * Revision           $Revision: 1.1 $
 * Release status     Experimental. Do not distribute. $State: Exp $
 *
 * Last modified on   $Date: 2002/01/10 14:18:51 $
 *               by   $Author: ijd $
 *
 * (c) Copyright Hewlett-Packard Company 2002
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *****************************************************************************/

// Package
///////////////
package examples.daml.ex1;


// Imports
///////////////
import java.io.*;
import java.util.*;
import javax.swing.*;

import com.hp.hpl.jena.daml.*;
import com.hp.hpl.mesa.rdf.jena.model.*;


/**
 * A primitive viewer class to illustrate one approach to viewing DAML ontologies.
 * The primary purpose of this code is to illustrate some of the features of the
 * DAML API, rather than to provide a definitively useful ontology viewer.
 *
 * @author Ian Dickinson, HP Labs (<a href="mailto:Ian_Dickinson@hp.com">email</a>)
 * @version CVS info: $Id: DAMLViewer.java,v 1.1 2002/01/10 14:18:51 ijd Exp $
 */
public class DAMLViewer
{
    // Constants
    //////////////////////////////////


    // Static variables
    //////////////////////////////////


    // Instance variables
    //////////////////////////////////

    /** The main viewer frame */
    private ViewerFrame m_viewFrame = null;

    /** Translation table for namespace prefixes */
    private Properties m_namespacePrefixes = new Properties();

    /** Show anon values? */
    private boolean m_showAnonymous = true;

    /** Flag to control whether import blocking is used */
    private boolean m_useImportBlocking = true;

    /** Flag to control whether imports are autoloaded */
    private boolean m_loadImports = true;

    /** The set of active inspectors */
    private Vector m_inspectors = new Vector();


    // Constructors
    //////////////////////////////////


    // External signature methods
    //////////////////////////////////

    /**
     * Main entry point. Optionally takes the URL of the ontology to view
     * as a parameter.
     *
     * @param args The arguments passed to the application from the jvm.
     */
    public static void main( String[] args ) {
        new DAMLViewer().go( args );
    }


    /**
     * Render a DAML name into something brief and readable
     */
    public String renderDAMLObjectName( Object o ) {
        if (o instanceof Resource) {
            return getDAMLObjectPresentation( (Resource) o );
        }
        else {
            return o.toString();
        }
    }


    /**
     * Answer true if we are showing the anonymous values
     */
    public boolean getShowAnonymous() {
        return m_showAnonymous;
    }


    /**
     * Set the flag to show anonymous values
     */
    public void setShowAnonymous( boolean showAnon ) {
        m_showAnonymous = showAnon;
    }


    /**
     * Answer true if we are blocking well known imports
     */
    public boolean getUseImportBlocking() {
        return m_showAnonymous;
    }


    /**
     * Set the flag to block well known imports
     */
    public void setUseImportBlocking( boolean showAnon ) {
        m_showAnonymous = showAnon;
    }


    /**
     * Answer true if we are loading imports
     */
    public boolean getLoadImports() {
        return m_showAnonymous;
    }


    /**
     * Set the flag to load imports
     */
    public void setLoadImports( boolean showAnon ) {
        m_showAnonymous = showAnon;
    }


    /**
     * Answer the viewer frame for the viewer
     */
    public ViewerFrame getFrame() {
        return m_viewFrame;
    }


    /**
     * Open an inspector on the given DAML value
     */
    public void inspect( DAMLCommon item ) {
        DAMLInspector i = null;

        // create the appropriate class of inspector for this object
        if (item instanceof DAMLClass) {
           i = new ClassInspector( (DAMLClass) item, this );
        }
        else if (item instanceof DAMLProperty) {
           i = new PropertyInspector( (DAMLProperty) item, this );
        }
        else if (item instanceof DAMLInstance) {
           i = new InstanceInspector( (DAMLInstance) item, this );
        }
        else {
            System.out.println( "Appropriate inspector not implemented yet! " );
        }

        // create an entry on the window menu
        if (i != null) {
            JMenuItem mnu = getFrame().addWindowMenuEntry( i );
            i.setWindowMenuEntry( mnu );
            i.showInspector();
            m_inspectors.add( i );
        }
    }


    /**
     * Close all inspector windows
     */
    public void closeInspectors() {
        for (Iterator i = m_inspectors.iterator();  i.hasNext(); ) {
            ((JFrame) i.next()).dispose();
        }
    }


    /**
     * Remove an inspector from the list
     */
    public void removeInspector( DAMLInspector i ) {
        m_inspectors.remove( i );
    }


    // Internal implementation methods
    //////////////////////////////////

    /**
     * Scan the arguments first
     */
    private void go( String[] args ) {
        int n = 0;
        boolean processingArgs = true;

        // scan the args to check for ones we know
        while (processingArgs  &&  n < args.length) {
            // -n introduces a file of namespace prefixes
            if (args[n].equals( "-n" )) {
                n++;
                if (n <= args.length) {
                    // load the namespace prefixes
                    loadNamespacePrefixes( args[n++] );
                }
            }
            else {
                // default case - stop processing args
                processingArgs = false;
            }
        }

        // start the viewer
        view( (args.length > n) ? args[n] : null );
    }


    /**
     * Build and show the main viewer frame
     */
    private void view( String uri ) {
        m_viewFrame = new ViewerFrame( this );

        // set the window size
        m_viewFrame.setSize( 600, 400 );
        m_viewFrame.setVisible( true );

        // load the given document, if any
        if (uri != null) {
            m_viewFrame.loadURL( uri );
        }
    }


    /**
     * Load a properties file containing the namespace prefixes
     */
    private void loadNamespacePrefixes( String nsFile ) {
        FileInputStream f = null;

        try {
            f = new FileInputStream( nsFile );
            m_namespacePrefixes.load( f );
        }
        catch (IOException e) {
            System.err.println( "Failed to read namespace prefix file: " + e );
            System.exit(1);
        }
        finally {
            if (f != null) {
                try {
                    f.close();
                }
                catch (IOException ignore) {}
            }
        }
    }


    // generating display names for DAML objects
    // TODO: this should probably be refactored out to a new class

    /**
     * Answer the presentation name for a DAML value
     */
    private String getDAMLObjectPresentation( Resource value ) {
        if (value.isAnon()) {
            // just say what sort of thing it is
            return "anonymous " + getDAMLValueTypePresentation( value );
        }
        else {
            // do we know a ns prefix for the namespace?
            String prefix = null;

            for (Iterator i = m_namespacePrefixes.entrySet().iterator(); i.hasNext(); ) {
                Map.Entry entry = (Map.Entry) i.next();
                if (entry.getValue().equals( value.getNameSpace() )) {
                    // found a matching prefix - we can return it
                    return (entry.getKey() + ":" + value.getLocalName());
                }
            }

            // to get here, we know no prefix - just return the URI
            return value.getURI();
        }
    }

    private String getDAMLValueTypePresentation( Resource value ) {
        if      (!(value instanceof DAMLCommon)) return "RDF resource";
        else if (value instanceof DAMLRestriction) return "restriction";
        else if (value instanceof DAMLClass) return "class";
        else if (value instanceof DAMLDataInstance) return "data instance";
        else if (value instanceof DAMLDatatype) return "datatype";
        else if (value instanceof DAMLDisjoint) return "disjoint axiom";
        else if (value instanceof DAMLList) return "list";
        else if (value instanceof DAMLOntology) return "ontology element";
        else if (value instanceof DAMLDatatypeProperty) return "datatype property";
        else if (value instanceof DAMLObjectProperty) return "object property";
        else {
            StringBuffer s = new StringBuffer( "instance " );
            String sep = "of ";

            for (Iterator i = ((DAMLCommon) value).getRDFTypes( false );  i.hasNext(); ) {
                s.append( sep );
                s.append( getDAMLObjectPresentation( (Resource) i.next() ));

                sep = ", ";
            }

            return s.toString();
        }
    }


    //==============================================================================
    // Inner class definitions
    //==============================================================================


}
