//--HEADER--
/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * $Id: WorkingVar.java,v 1.1 2001/09/19 09:28:24 afs Exp $
 */
//--HEADER--

package com.hp.hpl.jena.rdf.query  ;

import java.io.PrintWriter ;

/** Query engine working variable - different from variables in the query language.
 *  Working variables are settable values for holding intermediate results.
 */

// WorkingVar are polymorphic in that the value they are can change.
// In RDF, there is no support in the language itself for distinguishing
// strings from ints from booleans.  Hence all literals are strings
// which can be interpreted in the context of, say, a number.

// Settable extends Value

public class WorkingVar implements /*Value,*/ Printable, Settable, Cloneable
{
    boolean isSet = false ;

    boolean isInt = false ;
    boolean isBoolean = false ;
    boolean isLong = false ;
    boolean isURI = false ;
    boolean isString = false ;

    int valInt ;
    boolean valBoolean ;
    long valLong ;
    String valString ;
    String valURI ;


    public WorkingVar() { unset() ; }

    private void unset()
    {
        isSet = false ;
        // Throw away any old values

        if ( isString )
            valString = null ;

        if ( isURI )
            valURI = null ;

        isInt = false ;
        isBoolean = false ;
        isLong = false ;
        isURI = false ;
        isString = false ;
    }

    public void setInt(int i)         { unset() ; isSet = true ; isInt = true ; valInt = i ; }
    public void setLong(long l)       { unset() ; isSet = true ; isLong = true ; valLong = l ; }
    public void setBoolean(boolean b) { unset() ; isSet = true ; isBoolean = true ; valBoolean = b ; }
    public void setString(String s)   { unset() ; isSet = true ; isString = true ; valString = s ; }
    public void setURI(String uri)    { unset() ; isSet = true ; isURI = true ; valURI = uri ; }

    //public boolean isNumber()   { return isSet && (isInt || isLong) ; }
    public boolean isNumber()
    {
        forceInt() ;
        if ( ! isInt )
            forceLong() ;
         return isSet && (isInt || isLong) ;
    }


    public boolean isInt()      { forceInt() ;     return isSet && isInt ; }
    public boolean isLong()     { forceLong() ;    return isSet && isLong ; }
    public boolean isBoolean()  {                  return isSet && isBoolean ; }
    public boolean isString()   {                  return isSet && isString ; }
    public boolean isURI()      {                  return isSet && isURI ; }

    private void forceInt()
    {
        if ( ! isSet || isInt || ! isString ) return ;
        try {
            valInt = Integer.parseInt(valString) ;
            isInt = true ;
            isLong = true ;
            valLong = valInt ;
        } catch (NumberFormatException e) { return ; }
    }

    private void forceLong()
    {
        if ( ! isSet || isLong || ! isString ) return ;
        try {
            valLong = Long.parseLong(valString) ;
            isLong = true ;
        } catch (NumberFormatException e) { return ; }
    }


    public int getInt()
    {
        forceInt() ;
        if ( ! isSet || ! isInt ) throw new ValueException("Not an int: "+this) ;
        return valInt ;
    }

    public long getLong()
    {
        forceLong() ;
        if ( ! isSet || ! isLong ) throw new ValueException("Not a long: "+this) ;
        return valLong ;
    }

    public boolean getBoolean()
    {
        if ( ! isSet || ! isBoolean ) throw new ValueException("Not a boolean: "+this) ;
        return valBoolean ;
    }

    // No quoting or escape processing done
    public String getString()
    {
        if ( ! isSet ) return "<<unset>>" ;
        if ( isInt ) return Integer.toString(valInt) ;
        if ( isLong ) return Long.toString(valLong) ;
        if ( isURI ) return valURI ;
        if ( isBoolean ) return (valBoolean?"true":"false") ;
        if ( isString ) return valString ;

        return "<<unknown>>" ;
    }

    public String getURI()
    {
        if ( ! isSet || ! isURI ) throw new ValueException("Not a URI: "+this) ;
        return valURI ;
    }

    public String asInfixString() { return asQuotedString() ; }

    public String asQuotedString()
    {
        if ( ! isSet ) return "<<unset>>" ;
        if ( isInt ) return Integer.toString(valInt) ;
        if ( isLong ) return Long.toString(valLong) ;
        // Escaping needed
        if ( isURI ) return "<"+valURI+">" ;
        if ( isBoolean ) return (valBoolean?"true":"false") ;
        // Escaping needed
        if ( isString ) return "\""+valString+"\"" ;

        return "<<unknown>>" ;
    }

    public String asPrefixString() { return asStringWithType() ; }

    private String asStringWithType()
    {
        if ( ! isSet ) return "<<unset>>" ;
        if ( isInt ) return "int:"+Integer.toString(valInt) ;
        if ( isLong ) return "long:"+Long.toString(valLong) ;
        if ( isURI ) return "URI:"+valURI ;
        if ( isBoolean ) return "boolean:"+(valBoolean?"true":"false") ;
        if ( isString ) return "string:"+valString ;

        return "<<unknown>>" ;
    }

    // Print prefix notation (multiline) for debugging
    public void print(PrintWriter pw, int level)
    {
        QueryFormatUtils.indent(pw, level) ;
        pw.println(this.asPrefixString()) ;
    }

    public String asUnquotedString() { return getString() ; }
    public String toString() { return getString() ; }
}
