//--HEADER--
/*
 *  (c) Copyright Hewlett-Packard Company 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

/*
 * $Id: Literal.java,v 1.2 2001/12/05 10:29:38 afs Exp $
 */
//--HEADER--

package com.hp.hpl.jena.rdf.query.parser ;

import com.hp.hpl.jena.rdf.query.*;
import java.io.PrintWriter;

// A literal is an implementation of value that is created
// from the parsing process.


public class Literal extends SimpleNode implements Value, Expr
{
    boolean isSet = false ;

    boolean isInt = false ;
    boolean isBoolean = false ;
    boolean isLong = false ;
    boolean isURI = false ;
    boolean isString = false ;

    int valInt ;
    boolean valBoolean ;
    long valLong ;
    String valString ;
    String valURI ;

    // Constructors used by the parser
    Literal(int id) { super(id); }

    Literal(RDQLParser p, int id) { super(p, id); }


    protected void unset()
    {
        isSet = false ;
        // Throw away any old values

        if ( isString )
            valString = null ;

        if ( isURI )
            valURI = null ;

        isInt = false ;
        isBoolean = false ;
        isLong = false ;
        isURI = false ;
        isString = false ;
    }

    public Value eval(Query q, ResultBinding env)
    {
        if ( ! isSet )
            throw new EvalFailureException("Literal value not set") ;

         return this ;
    }

    public boolean isSet() { return isSet ; }

    public boolean isNumber()   { return isSet && (isInt || isLong) ; }
    public boolean isInt()      { return isSet && isInt ; }
    public boolean isLong()     { return isSet && isLong ; }
    public boolean isBoolean()  { return isSet && isBoolean ; }
    public boolean isString()   { return isSet && isString ; }
    public boolean isURI()      { return isSet && isURI ; }

    // NB Protected - and should be set-once
    protected void setInt(int i)         { unset() ; isSet = true ; isInt = true ; valInt = i ; }
    protected void setLong(long l)       { unset() ; isSet = true ; isLong = true ; valLong = l ; }
    protected void setBoolean(boolean b) { unset() ; isSet = true ; isBoolean = true ; valBoolean = b ; }
    protected void setString(String s)   { unset() ; isSet = true ; isString = true ; valString = s ; }
    protected void setURI(String uri)    { unset() ; isSet = true ; isURI = true ; isString = true ; valURI = uri ; valString = uri ; }

    public int getInt()
    {
        if ( ! isSet || ! isInt ) throw new ValueException("Not an int: "+this) ;
        return valInt ;
    }

    public long getLong()
    {
        if ( ! isSet || ! isLong ) throw new ValueException("Not a long: "+this) ;
        return valLong ;
    }

    public boolean getBoolean()
    {
        if ( ! isSet || ! isBoolean ) throw new ValueException("Not a boolean: "+this) ;
        return valBoolean ;
    }

    public String getString()
    {
        if ( ! isSet || ! isString ) throw new ValueException("Not a string: "+this) ;
        return valString ;
    }

    public String getURI()
    {
        if ( ! isSet || ! isURI ) throw new ValueException("Not a URI: "+this) ;
        return valURI ;
    }


    // In all these stringification operations, order matters e.g. URI before string
    public String asQuotedString()
    {
        if ( ! isSet ) return "literal:unset" ;
        if ( isInt ) return Integer.toString(valInt) ;
        if ( isLong ) return Long.toString(valLong) ;
        if ( isBoolean ) return (valBoolean?"true":"false") ;
        // Escaping needed
        if ( isURI ) return "<"+valURI+">" ;
        // Escaping needed
        if ( isString ) return "\""+valString+"\"" ;

        return "literal:unknown" ;
    }

    // Does not quote strings or URIs
    public String asUnquotedString()
    {
        if ( ! isSet ) return "literal:unset" ;
        if ( isInt ) return Integer.toString(valInt) ;
        if ( isLong ) return Long.toString(valLong) ;
        if ( isBoolean ) return (valBoolean?"true":"false") ;
        if ( isURI ) return valURI ;
        if ( isString ) return valString ;

        return "literal:unknown" ;
    }

    public String asInfixString() { return asQuotedString() ; }

    public String asPrefixString()
    {
        if ( ! isSet ) return "literal:unset" ;
        if ( isInt ) return "int:"+Integer.toString(valInt) ;
        if ( isLong ) return "long:"+Long.toString(valLong) ;
        if ( isBoolean ) return "boolean:"+(valBoolean?"true":"false") ;
        if ( isURI ) return "URI:"+valURI ;
        if ( isString ) return "string:"+valString ;

        return "literal:unknown" ;
    }

    // Print prefix notation (multiline) for debugging
    public void print(PrintWriter pw, int level)
    {
        QueryFormatUtils.indent(pw, level) ;
        pw.println(this.asPrefixString()) ;
    }

    // This is used in the filtering stage to get values for testing - must be unquoted.
    public String toString()
    {
        return asUnquotedString() ;
    }
}
