/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * RDFLoader.java Uses David Megginson's RDF Filter
 *                to parse an xml file and add to a model
 *
 * Created on 20 August 2000, 10:10
 */

package com.hp.hpl.mesa.rdf.jena.common;



import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;

import com.megginson.sax.rdf.RDFHandler;
import com.megginson.sax.rdf.RDFReader;

import org.xml.sax.Attributes;
import org.xml.sax.ContentHandler;
//import org.xml.sax.Parser;
import org.xml.sax.XMLReader;
import org.xml.sax.SAXParseException;
//import org.xml.sax.helpers.ParserAdapter;
//import org.xml.sax.helpers.ParserFactory;
import org.xml.sax.helpers.XMLReaderFactory;

import java.net.URL;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.HashMap;

/** Uses David Megginson's RDF Filter to load an XML serialization into a model.
 *
 * <p>Uses the Sax2 parser specified by the org.xml.sax.driver system property.
 *    If this is not set, uses the Apache Xerces parser by default.</p>
 *
 * @author  bwm
 * @version  Release='$Name:  $' Revision='$Revision: 1.3 $' Date='$Date: 2001/07/04 18:40:33 $' 
 */
public class RDFXMLReader extends Object implements RDFHandler,
      com.hp.hpl.mesa.rdf.jena.model.RDFReader {
    
    // next available error code = 3
        
    static final String saxDriverPropName = "org.xml.sax.driver";
    static final String defaultSaxDriverProp = 
                                         "org.apache.xerces.parsers.SAXParser";
    static final String namespacePrefixPropName =
                          "http://org.xml.sax/features/namespace-prefixes";
    static final String namespacePropName = 
                                       "http://org.xml.sax.features/namespace";
    
    RDFErrorHandler errorHandler = new RDFDefaultErrorHandler();
    
    XMLContentHandler contentHandler = new XMLContentHandler();
    RDFReader rdfReader;  // RDFFilter instance
    
    int subjectType;
    String subject;
    String predicate;
    String language;
    String base;

    Model model;                            // the model being loaded
    HashMap anonResources = new HashMap();

    RDFXMLReader() {} 

    public void read(Model model, Reader reader, String base)
      throws RDFException {
        
        this.model = model;
        this.base = base;
        
        // m&s states that the base to use is the base URI of the
        // document - so we have to throw away any fragment id's
        
        int i = base.lastIndexOf('#');
        if (i != -1) {
            base = base.substring(0,i);
        }
        
        try {
        
            String parserClassName = System.getProperty(saxDriverPropName,
                                                        defaultSaxDriverProp);    
            XMLReader xmlReader = 
                XMLReaderFactory.createXMLReader(parserClassName);
            rdfReader = new RDFReader(xmlReader);
            rdfReader.setRDFHandler(this);
            rdfReader.setContentHandler(getContentHandler());            
            rdfReader.setErrorHandler(new SaxErrorHandler(errorHandler));        
            rdfReader.readRDF(reader);
        } catch (Exception e) {
            errorHandler.error(e);
        }
    }
    
    public void read(Model model, String url) throws RDFException {
        try {
            read(model, 
                 new InputStreamReader(((new URL(url))).openStream()), url);
        } catch (Exception e) {
            throw new RDFException(e);
        }
    }

    
/** Set a property to control the reader's behaviour.
 *
 * <p>This reader currently recognises no properties.  Invoking this
 * method always causes an <CODE>UNKNOWNPROPERTY RDFException</CODE>
 * to be raised.</p>?
 * @param propName The name of the property to be set
 * @param propValue The new value of the property
 * @throws RDFException Throws <CODE>UNKNOWNPROPERTY RDFException</CODE> if the
 * property name is not recognised
 * @return the previous value of the property
 */        
    public Object setProperty(String propName,Object propValue)
      throws RDFException {
        errorHandler.error(new RDFException(RDFException.UNKNOWNPROPERTY));
        return null;
    }

    public RDFErrorHandler setErrorHandler(RDFErrorHandler errHandler) {
        RDFErrorHandler old = this.errorHandler;
        this.errorHandler = errHandler;
        if (rdfReader!=null)
          rdfReader.setErrorHandler(new SaxErrorHandler(errorHandler));
        return old;
    }
    
    
    protected XMLContentHandler getContentHandler() {
        return contentHandler;
    }
    
    public void literalStatement(int subjectType, String subject,
                                  String predicate, String object,
                                  String language) {
      literalStatement(subjectType, subject, predicate, object, language,
                       false);
    }
    
    public void literalStatement(int subjectType, String subject,
                                  String predicate, String object,
                                  String language, boolean wellFormed) {
        if (language==null) language="";
        //
        // This is Jeremy's idea.  He is concerned about normalising
        // newlines in literals across different platoforms.
        //
        object = translateCRLFtoNL(object);
        try {
            if (subjectType==SUBJECT_URI) {
              model.add(model.createResource(translateURI(subject)),
                        model.createProperty(translateURI(predicate)),
                        object, language, wellFormed);
            } else if (subjectType==SUBJECT_GENERATED) {
                model.add(getAnonResource(subject),
                          model.createProperty(translateURI(predicate)),
                          object, language, wellFormed);
            } else {
                ErrorHelper.logWarning(
                              "subject type not supported: " + subject);
            }
        } catch (RDFException e) {
            ErrorHelper.logInternalError("RDFLoader", 1, e);
        }
    }
    
    public void resourceStatement(int subjectType, String subject, 
                                    String predicate, String object) {
        try {
            Resource s;
            Resource o;
            if (subjectType==SUBJECT_URI) {
                s = model.createResource(translateURI(subject));
            } else if (subjectType==SUBJECT_GENERATED) {
                s = getAnonResource(subject);
            } else {
                ErrorHelper.logWarning(
                               "subject type not supported: " + subject);
                return;
            }
            
                                              // FIXME: kludge here to deal
                                              // with RDFFilter problem.  RDFF
                                              // doesn't tell us if the object
                                              // is an anon resource id
                                              // so we judge by looking at
                                              // the first character.
            if (object.startsWith("{")) {
                o = getAnonResource(object);
            } else {
                o = model.createResource(translateURI(object));
            }
            model.add(s, model.createProperty(translateURI(predicate)), o);
        } catch (RDFException e) {
            ErrorHelper.logInternalError("RDFLoader", 2, e);
        }
    }
    
    public void startXMLStatement(int subjectType, String subject,
                                   String predicate, String language) {
        this.subjectType = subjectType;
        this.subject = subject;
        this.predicate = predicate;
        this.language = language;
        contentHandler.activate();
    }
    
    public void endXMLStatement()
    {
        literalStatement(subjectType, subject, predicate, 
                            contentHandler.deActivate(), language, true);        
    }

    protected Resource getAnonResource(String id) throws RDFException {
        Resource resource;
        
        resource = (Resource) anonResources.get(id);
        if (resource==null) {
            resource = model.createResource();
            anonResources.put(id, resource);
        }
        return resource;
    }
    
    protected String translateURI(String uri) {
        if (uri.length() == 0) {
            return base;
        } else if (uri.charAt(0) == '#') {
            return base + uri;
        } else {
            return uri;
        }
    }
    
    private static String translateCRLFtoNL(String in) {
        // check for the common case - no CR
        if (in.indexOf('\r') == -1) {
            return in;
        }
        // there is at least one carriage return
        char buf[] = new char[in.length()];
        in.copyValueOf(buf);
        int j = 0;
        for (int i=0; i<buf.length-1; i++) { // take care not to overrun end
            if (buf[i] != '\r' || buf[i+1] != '\n') {
                buf[j++] = buf[i];
            }
        }
        buf[j++] = buf[buf.length-1];        // don't forget the last char
        return new String (buf, 0, j);
    }
}

class XMLContentHandler implements ContentHandler {
    
    String content;                     // accumulator of the XML content
    boolean active;                     // whether the accumulator is active
    boolean elementEmpty=false;         // whether current element is empty
    
    public void activate() {
        active=true;
        content="";
    }
    
    public String deActivate() {
        String result = content;
        active = false;
        content = null;
        return result;
    }
    
    public void characters(char[] cb, int start, int length) {
        if (!active) return;
        closeOpenElement();
        content = content + 
            Util.substituteStandardEntities(String.valueOf(cb, start, length));
    }
    
    public void ignorableWhitespace(char[] cb, int start, int length) {
        characters(cb, start, length);
    }    
    
    public void startElement(String nameSpaceURI, String localName,
                               String qname, Attributes atts) {
                                
       if (!active) return;
       closeOpenElement();
       content = content + "<" + qname;
       for (int i=0; i<atts.getLength(); i++) {
           content = content
                       + " " + atts.getQName(i) + "=\""
                       + Util.replace(atts.getValue(i), "\"", "&quote;") + "\"";
       }
       elementEmpty = true;
    }
    
    public void endElement(String namespaceURI, String localName,
                             String qname) {
        if (!active) return;
        if (elementEmpty) {
            content = content + "/>";
        } else {
            content = content + "</" + qname + ">";
        }
        elementEmpty = false;
    }
    
    public void processingInstruction(String target, String data) {
        if (!active) return;
        closeOpenElement();
        content = content + "<?" + target + " " + data + "?>";
    }
    
    public void startDocument() {}
    public void endDocument() {}
    public void startPrefixMapping(String prefix, String uri) {}
    public void endPrefixMapping(String prefex) {}
    public void skippedEntity(java.lang.String name) {}
    public void setDocumentLocator(org.xml.sax.Locator locator) {}
    
    void closeOpenElement() {
        if (elementEmpty) {
            content = content + ">";
            elementEmpty = false;
        }
    }
}
    
class SaxErrorHandler extends Object implements org.xml.sax.ErrorHandler {
    RDFErrorHandler errorHandler;
    
    public SaxErrorHandler(RDFErrorHandler errorHandler) {
        this.errorHandler = errorHandler;
    }
        
    public void error(SAXParseException e) {
        errorHandler.error(e);
    }
        
    public void warning(SAXParseException e) {
        errorHandler.warning(e);
    }
        
    public void fatalError(SAXParseException e) {
        errorHandler.fatalError(e);
    }
}