/*
 *  (c) Copyright Hewlett-Packard Company 2000, 2001
 *  All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.


 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * RDFWriter.java
 *
 * Created on 29 July 2000, 07:28
 */


package com.hp.hpl.mesa.rdf.jena.common;



import com.hp.hpl.mesa.rdf.jena.model.*;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDF;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RDFS;
import com.hp.hpl.mesa.rdf.jena.vocabulary.DC;
import com.hp.hpl.mesa.rdf.jena.vocabulary.RSS;


import java.io.PrintWriter;
import java.io.Writer;
import java.util.Properties;
import java.util.Enumeration;
import java.util.HashMap;


/** Writes out an XML serialization of a model.
 *
 * @author  bwm
 * @version   Release='$Name:  $' Revision='$Revision: 1.5 $' Date='$Date: 2002/01/09 16:55:34 $'
 */
public class RDFXMLWriter extends Object implements RDFWriter {


    Properties nsMap     = new Properties();
    String xmlBase       = null;
    HashMap    anonMap   = new HashMap();
    int        anonCount = 0;
    static String RDFNS  = RDF.getURI();
    RDFErrorHandler errorHandler = new RDFDefaultErrorHandler();


    public RDFXMLWriter() {
      nsMap.setProperty(RDF.getURI(),  "rdf");
      nsMap.setProperty(RDFS.getURI(), "rdfs");
      nsMap.setProperty(DC.getURI(),   "dc");
      nsMap.setProperty(RSS.getURI(),  "rss");
      nsMap.setProperty("http://www.daml.org/2001/03/daml+oil.daml#", "daml");
    }


    /** Write out an XML serialization of a model.
     * @param model the model to be serialized
     * @param writer the PrintWriter to receive the serialization
     * @throws IOException if an io error occurs
     * @throws RDFException if any other exception occurs
     */
    public void write(Model model, java.io.Writer writer, String base)
    throws RDFException {
        try {
            PrintWriter pw;
            if (writer instanceof PrintWriter) {
                pw = (PrintWriter) writer;
            } else {
                pw = new PrintWriter(writer);
            }
            listNameSpaces(model);
            writeRDFHeader(model, pw, base);
            writeRDFStatements(model, pw);
            writeRDFTrailer(pw, base);
            writer.flush();
        } catch (Exception e) {
            errorHandler.error(e);
        }
    }


    public void setNsPrefix(String prefix, String ns) {
        nsMap.setProperty(ns, prefix);
    }


    public RDFErrorHandler setErrorHandler(RDFErrorHandler errHandler) {
        RDFErrorHandler old = this.errorHandler;
        this.errorHandler = errHandler;
        return old;
    }


    protected void listNameSpaces(Model model) throws RDFException {
        NsIterator nsIter = model.listNameSpaces();
        int count = 0;
        String ns;
        String sysprop;
        while (nsIter.hasNext()) {
            ns = nsIter.next();
            sysprop = System.getProperty(RDFWriter.NSPREFIXPROPBASE + ns);
            if (sysprop != null) {
                nsMap.setProperty(ns, sysprop);
            } else if (nsMap.getProperty(ns) == null) {
                nsMap.put(ns, "NS" + Integer.toString(count++));
            }
        }
    }


    protected void writeRDFHeader(Model model, PrintWriter writer, String base)
      throws RDFException {


        NsIterator nsIter = model.listNameSpaces();
        String ns;


        writer.println("<" + nsPrefix(RDFNS) + ":RDF");
        writer.println("  xmlns:" + nsPrefix(RDFNS)
                                    + "='" + RDFNS + "'");
        if (null != xmlBase && xmlBase.length() > 0) {


          writer.println("  xml:base=\"" + xmlBase + "\"");
        }
        while (nsIter.hasNext()) {
            ns = (String) nsIter.next();
            // another kludge for rdf namespace
            if (!ns.equals(RDFNS)) {
                writer.println("  xmlns:" + nsPrefix(ns) + "='" + ns + "'");
            }
        }
        writer.println(" >");
    }


    protected void
        writeRDFStatements(Model model, PrintWriter writer)
            throws RDFException {
        ResIterator rIter = model.listSubjects();
        while (rIter.hasNext()) {
            writeRDFStatements(model, rIter.next(), writer);
        }
    }


    protected void writeRDFTrailer(PrintWriter writer, String base) {
        writer.println("</" + nsPrefix(RDFNS) + ":RDF>");
    }


    protected void
      writeRDFStatements(Model model, Resource subject, PrintWriter writer)
        throws RDFException {
        StmtIterator sIter =
          model.listStatements(new SelectorImpl(subject, null, (RDFNode) null));


        writeDescriptionHeader(subject, writer);
        if ( (subject instanceof Statement)
           && ! model.contains((Statement)subject)) {
            // an unasserted reified statement
            writeReifiedProperties((Statement) subject, writer);
        }
        while (sIter.hasNext()) {
            writePredicate(sIter.next(), writer);
        }
        writeDescriptionTrailer(writer);


        // if the subject of subject is a reified statement not in the model
        // need to write it out too
        if (subject instanceof Statement) {
            Resource innerSubject = ((Statement)subject).getSubject();
            if (innerSubject instanceof Statement
             && ! model.contains((Statement) innerSubject)) {
                writeRDFStatements(model, innerSubject, writer);
            }
            RDFNode innerObject = ((Statement)subject).getObject();
            if (innerObject instanceof Statement
             && ! model.contains((Statement) innerObject)) {
                writeRDFStatements(model, (Resource) innerObject, writer);
            }
        }


    }


    protected void writeDescriptionHeader(Resource subject, PrintWriter writer)
            throws RDFException {
        writer.print("  <" + nsPrefix(RDFNS) + ":Description ");
        writeResourceId(subject, writer);
        writer.println(">");
    }


    protected void
        writePredicate(Statement stmt, PrintWriter writer)
            throws RDFException {


        Property predicate = stmt.getPredicate();
        RDFNode object = stmt.getObject();


        writer.print("    <" + transNS(predicate));


        if (stmt.isReified()) {
            writer.print(" " + nsPrefix(RDFNS) + ":ID='" + anonId(stmt) + "'");
        }


        if (object instanceof Resource) {
            writer.print(" ");
            writeResourceReference(((Resource)object), writer);
            writer.println("/>");
        } else {
            writeLiteral((Literal) object, writer);
            writer.println("</" + transNS(predicate) + ">");
        }
    }


    protected void writeDescriptionTrailer(PrintWriter writer) {
        writer.println("  </" + nsPrefix(RDFNS) + ":Description>");
    }


    protected void writeReifiedProperties(Statement stmt, PrintWriter writer)
      throws RDFException {
        writer.println("    <" + nsPrefix(RDFNS) + ":type "
                               + nsPrefix(RDFNS) + ":resource='"
                       + RDFNS + "Statement'/>");
        writer.print("    <" + nsPrefix(RDFNS) + ":subject ");
        writeResourceReference(stmt.getSubject(), writer);
        writer.println("/>");
        writer.println("    <" + nsPrefix(RDFNS) + ":predicate "
                               + nsPrefix(RDFNS) + ":resource='"
                             + stmt.getPredicate().getURI() + "'/>");
        writer.print("    <" + nsPrefix(RDFNS) + ":object ");


        RDFNode object = stmt.getObject();
        if (object instanceof Resource) {
            writeResourceReference((Resource) stmt.getObject(), writer);
            writer.println("/>");
        } else {
            writeLiteral((Literal) object, writer);
            writer.println("</" + nsPrefix(RDFNS) + ":object>");
        }
    }


    protected void writeResourceId(Resource r, PrintWriter writer)
      throws RDFException {
        writer.print(nsPrefix(RDFNS) + ":about='");
        if (r.isAnon()) {
            writer.print("#" + anonId(r));
        } else {
            writer.print(r.getURI());
        }
        writer.print("'");
    }


    protected void writeResourceReference(Resource r, PrintWriter writer)
      throws RDFException {
        writer.print(nsPrefix(RDFNS) + ":resource='");
        if (r.isAnon()) {
            writer.print("#" + anonId(r));
        } else {
            writer.print(r.getURI());
        }
        writer.print("'");
    }


    protected void writeLiteral(Literal l, PrintWriter writer) {
        String lang = l.getLanguage();
        if (! lang.equals("")) {
            writer.print(" xml:lang=\'" + lang + "'");
        }
        if (l.getWellFormed()) {
            writer.print(" " + nsPrefix(RDFNS) + ":parseType='Literal'>");
            writer.print(l.toString());
        } else {
            writer.print(">");
            writer.print(Util.substituteStandardEntities(l.toString()));
        }
    }



    protected String anonId(Resource r) throws RDFException {
        String result = (String) anonMap.get(r.getId());
        if (result == null) {
            result = "A" + Integer.toString(anonCount++);
            anonMap.put(r.getId(), result);
        }
      return result;
    }


    protected String transNS(Property p) {
      return nsPrefix(p.getNameSpace()) + ":" + p.getLocalName();
    }


    protected String nsPrefix(String ns) {
        return nsMap.getProperty(ns);
    }



    /**
     * Sets the XML base if required.  Xmlbase is assumed to be a string.
     *
     * @param propName Must be "xmlbase"
     * @param propValue The string (URI) to set for the xmlbase object.
     * @return the old value for this property, or <code>null</code>
     * if no value was set.
     */
    public Object setProperty(String propName, Object propValue) throws RDFException {
        if ( propName.equalsIgnoreCase("xmlbase") ) {
            String result = xmlBase;
            xmlBase = (String) propValue;
            return result;
        } else {
            // Copied from Brian! This is overloading the error condition!
            // bwm: true - too true
            throw new RDFException(RDFException.UNKNOWNPROPERTY);
        }
    }
} 