/* -*-Java-*-

*******************************************************************

*

* File:         DriverGenericGeneric.java

* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/DriverGenericGeneric.java,v 1.6 2002/04/26 15:11:13 der Exp $

* Author:       Dave Reynolds, HPLabs, Bristol

* Created:      18/9/01

* Modified:     $Date: 2002/04/26 15:11:13 $

* Status:       Experimental (Do Not Distribute)

*

* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.

* [See end of file]

*********************************************************************/



//=======================================================================

// Package

package com.hp.hpl.mesa.rdf.jena.rdb;



//=======================================================================

// Imports

import com.hp.hpl.mesa.rdf.jena.common.*;

import com.hp.hpl.mesa.rdf.jena.model.*;

import java.io.*;

import java.sql.*;

import java.util.*;

import java.util.zip.CRC32;

import com.hp.hpl.jena.util.Log;



//=======================================================================

/**

* Base database driver for implementing ModelRDB and StoreRDB.

* Different drivers are needed for different databases and different

* layout schemes.

* <p>

* This driver is a base implemention from which database-specific

* drivers can inherit. It is not generic in the sense that it will work

* on any minimal SQL store and so should be treated as if it were

* an abstract class.

* <p>The SQL statements which implement each of the functions are

* loaded in a separate file etc/[layout]_[database].sql from the classpath.

* See {@link SQLCache SQLCache documentation} for more information on the

* format of this file.

*

* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>

* @version $Revision: 1.6 $ on $Date: 2002/04/26 15:11:13 $

*/



public class DriverGenericGeneric implements IRDBDriver {



//=======================================================================

// Cutomization variables



    /** The class name of the database driver */

    protected String DRIVER_NAME = "java.lang.Class";     // Dummy - needs replacing in subclasses



    /** The name of the database this driver supports */

    protected String LAYOUT_TYPE = "Generic";



    /** The name of the layout type this driver supports */

    protected String DATABASE_TYPE = "Generic";



    /** The maximum size of literals that can be indexed */

    protected int MAX_LITERAL = 32000;



    /** The name of the sql definition file for this database/layout combo */

    protected String SQL_FILE = "etc/generic_generic.sql";



    /** The SQL type to use for storing ids (compatible with wrapDBID) */

    protected String ID_SQL_TYPE = "INTEGER";



    /** Set to true if the insert operations already check for duplications */

    protected boolean SKIP_DUPLICATE_CHECK = false;



    /** Set to true if the insert operations allocate object IDs themselves */

    protected boolean SKIP_ALLOCATE_ID = false;



    /** Set to true if the insert operations should be done using the "proc" versions */

    protected boolean INSERT_BY_PROCEDURE = false;



    /** Set to true to enable cache of pre-prepared statements */

    protected boolean CACHE_PREPARED_STATEMENTS = true;



//=======================================================================

// Internal variables



    /** The packaged jdbc connection to the database */

    protected IDBConnection m_dbcon = null;



    /** The table of sql driver statements */

    protected SQLCache m_sql = null;



    /** Table of stores using this driver */

    protected List m_clientStores = new ArrayList();



    /** Namespace id used to flag anonymous resources - fixed during database init */

    protected IDBID m_anonNSId = new DBIDInt(0);



    /** Namesapce used to flag anonymous resources */

    public final static String ANON_NAMESPACE = "_";



    /** The name of the default sql definition file */

    protected final static String SQL_DEFAULT_FILE = "etc/generic_generic.sql";



    /** table of default sql statements */

    protected static Properties m_defaultSQL = null;



    /** The size of allocation blocks for the namespace cache */

    protected final static int NS_BLOCKSIZE = 50;



    /** Cache of all id-namespace mappings */

    protected Map m_namespaceMap = new HashMap(NS_BLOCKSIZE);



    /** Reverse cache of namespace-id mappings */

    protected Map m_namespaceIDMap = new HashMap(NS_BLOCKSIZE);



//=======================================================================

// Constructors and accessors



    /**

     * Create a bare instance of the driver. It is not functional until a

     * database connection has been supplied via setConnection.

     */

    public DriverGenericGeneric() {

        cacheNamespaceId(ANON_NAMESPACE, m_anonNSId);       // SQL code will pre-insert this const

    }



//=======================================================================

// Database operations



    /**

     * Link an existing instance of the driver to a specific jdbc connection

     */

    public void setConnection(IDBConnection dbcon) throws RDFRDBException {

        m_dbcon = dbcon;

        try {

            m_sql = new SQLCache(SQL_FILE, getDefaultSQL(), dbcon, ID_SQL_TYPE);

            m_sql.setCachePreparedStatements(CACHE_PREPARED_STATEMENTS);

        } catch (IOException e) {

            throw new RDFRDBException("Internal error loading sql defintion file: " + SQL_FILE, e);

        }

    }



    /**

     * Supporting function to load the table of default SQL operations

     */

    protected Properties getDefaultSQL() throws IOException {

        if (m_defaultSQL == null) {

            m_defaultSQL = SQLCache.loadSQLFile(SQL_DEFAULT_FILE, null, ID_SQL_TYPE);

        }

        return m_defaultSQL;

    }



    /**

     * Close the databse connection

     * @throws RDFRDBException if there is an access problem

     */

    public void close() throws RDFRDBException {

        try {

            m_dbcon.close();

            m_sql.close();

        } catch (SQLException e) {

            throw new RDFRDBException("Problem closing RDB driver", e);

        }

    }



    /**

     * Return the jdbc connection to the database, opening it if necessary

     */

    public Connection getConnection() throws RDFRDBException {

        try {

            return m_dbcon.getConnection();

        } catch (SQLException e) {

            throw new RDFRDBException("Problem opening database connection: " + e);

        }

    }



    /**

     * Return the SQLCache used for database commands.

     */

    public SQLCache getSQLCache() {

        return m_sql;

    }



    /**

     * Check that the database has a legal set of RDF tables.

     * This is not a full validation check simply a test for the existence of some

     * key tables as evidence of at least attempted formatting.

     * @return true if the database seems to be legally formated.

     * @throws RDFRDBException typtically due to internal programming errors, might trap

     * and change to logging operations in the future.

     */

    public boolean isDBFormatOK() throws RDFRDBException {

        try {

            Connection con = getConnection();

            DatabaseMetaData dbmd = con.getMetaData();

            String[] tableTypes = { "TABLE" };

            ResultSet alltables = dbmd.getTables(null, null, "RDF%", tableTypes);

            List tablesPresent = new ArrayList(10);

            while (alltables.next()) {

                tablesPresent.add(alltables.getString("TABLE_NAME").toUpperCase());

            }

            alltables.close();

            boolean ok = true;

            ok &= tablesPresent.contains("RDF_RESOURCES");

            ok &= tablesPresent.contains("RDF_STATEMENTS");

            ok &= tablesPresent.contains("RDF_LAYOUT_INFO");

            return ok;

        } catch (SQLException e1) {

            throw new RDFRDBException("Internal SQL error in driver", e1);

        }

    }



    /**

     * Initialise a database ready to store RDF tables.

     * This version uses a simple generic representation based a statement table and

     * a resource table, equivalent to the edge-based method of XML storage.

     * @throws RDFRDBException typtically due to internal programming errors, might trap

     * and change to logging operations in the future.

     */

    public void formatDB() throws RDFRDBException {

        try {

            m_sql.runSQLGroup("initDBtables");

            m_sql.runSQLGroup("initDBgenerators");

            m_sql.runSQLGroup("initDBprocedures");

            m_dbcon.addProperty(IDBConnection.DATABASE_TYPE, DATABASE_TYPE);

            m_dbcon.addProperty(IDBConnection.LAYOUT_TYPE, LAYOUT_TYPE);

            m_dbcon.addProperty(IDBConnection.DRIVER_CLASSNAME, this.getClass().getName());

            saveProperties();

        } catch (SQLException e) {

            com.hp.hpl.jena.util.Log.warning("Problem formatting database", e);

            throw new RDFRDBException("Failed to format database", e);

        }

    }



    /**

     * Remove all RDF information from a database.

     * This will only work if there are no other clients us the DB so is mostly to

     * simplify testing that a practical tool. Partial DB config or cleaning can leave

     * the DB in a strange state hence it will continue with each cleanup phase even

     * if earlier phases have generated errors and erros are only logged, not propagated

     * as exceptions.

     */

    public void cleanDB() {

        Exception err = null;

        try {m_sql.runSQLGroup("cleanDBprocedures");} catch (SQLException e) {

            Log.finest("Problem during database clean up", e);  // Log problem but contine

        }

        try {m_sql.runSQLGroup("cleanDBtables");    } catch (SQLException e) {

            Log.finest("Problem during database clean up", e);  // Log problem but contine

        }

        try {m_sql.runSQLGroup("cleanDBgenerators");} catch (SQLException e) {

            Log.finest("Problem during database clean up", e);  // Log problem but contine

        }

    }



//=======================================================================

// Support for loading/restoring configuration parameters from a DBConnection



    /**

     * Load a set of configuration parameters into the driver.

     */

     public void loadProperties(Properties props) {

        // Could be implemented more elegantly, in particular note confusing

        // reversal of partity on the SKIP_* flags - historical mess.

        String val;

        if ((val = props.getProperty("jdbcDriver")) != null)  DRIVER_NAME = val;

        if ((val = props.getProperty("sqlFile")) != null)     SQL_FILE = val;

        if ((val = props.getProperty("layout")) != null)      LAYOUT_TYPE = val;

        if ((val = props.getProperty("database")) != null)    DATABASE_TYPE = val;

        if ((val = props.getProperty("idType")) != null)      ID_SQL_TYPE = val;

        if ((val = props.getProperty("maxLiteral")) != null)  MAX_LITERAL = Integer.parseInt(val);

        if ((val = props.getProperty("allocateID")) != null)  SKIP_ALLOCATE_ID = !val.equals("true");

        if ((val = props.getProperty("checkDuplicates")) != null) SKIP_DUPLICATE_CHECK = !val.equals("true");

        if ((val = props.getProperty("insertByProcedure")) != null) INSERT_BY_PROCEDURE = val.equals("true");

        // Cache control is independently variable so don't restore

        //if ((val = props.getProperty("cachePreparedStatements")) != null) CACHE_PREPARED_STATEMENTS = val.equals("true");

     }



     /**

      * Store the configuration parameters onto the database layout table in the attached database

      */

     public void saveProperties() throws SQLException {

        m_dbcon.addProperty("jdbcDriver",   DRIVER_NAME);

        m_dbcon.addProperty("sqlFile",      SQL_FILE);

        m_dbcon.addProperty("layout",       LAYOUT_TYPE);

        m_dbcon.addProperty("database",     DATABASE_TYPE);

        m_dbcon.addProperty("idType",       ID_SQL_TYPE);

        m_dbcon.addProperty("maxLiteral",   Integer.toString(MAX_LITERAL));

        m_dbcon.addProperty("allocateID",      SKIP_ALLOCATE_ID ? "false" : "true");

        m_dbcon.addProperty("checkDuplicates", SKIP_DUPLICATE_CHECK ? "false" : "true");

        m_dbcon.addProperty("insertByProcedure", INSERT_BY_PROCEDURE ? "true" : "false");

        m_dbcon.addProperty("cachePreparedStatements", CACHE_PREPARED_STATEMENTS ? "true" : "false");

     }



     /**

      * Printable name for the driver configuration

      */

     public String toString() {

        return "jena.rdb.Driver" + LAYOUT_TYPE + DATABASE_TYPE + "(" + SQL_FILE + ")";

     }



//=======================================================================

// Store operations



    /**

     * Register a store as a user of this driver.

     * One driver can support multiple store instances.

     */

    public synchronized void attachStore(StoreRDB store) {

        m_clientStores.add(store);

    }



    /**

     * Notify that a store is no longer using this driver.

     */

    public synchronized void detachStore(StoreRDB store) {

        m_clientStores.remove(store);

        if (m_clientStores.isEmpty()) {

            // no more clients left, clean up the databsae connection

            try {

                close();

            } catch (RDFRDBException e) {

                Log.warning("Error which cleaning up RDB store", e);

            }

        }

    }



    /**

     * Register a statement in the database.

     * @param subjId database index of the subject entry

     * @param predId database index of the predicate entry

     * @param objId database index of the object entry

     * @param isLiteral if the object is a literal rather than a resource

     */

    public void addStatement(com.hp.hpl.mesa.rdf.jena.model.Statement s, ModelRDB m) throws RDFException {

        // Check if is already asserted

        if (!SKIP_DUPLICATE_CHECK) {

            if (isStatementPresent(s.getSubject(), s.getPredicate(), s.getObject(), m)) return;

        }

        // First ensure the components are all added

        IDBID predId = addResource(s.getPredicate());

        IDBID subjId = addResource(s.getSubject());

        RDFNode obj = s.getObject();

        IDBID objId = null;

        boolean isLiteral = obj instanceof Literal;

        if (isLiteral) {

            objId = addLiteral((Literal)obj);

        } else {

            objId = addResource((Resource)obj);

        }

        // Now add the statement itself

        try {

            Object[] args = new Object[] {subjId.getID(),

                                          predId.getID(),

                                          objId.getID(),

                                          wrapFlag(isLiteral),

                                          m.getDBID().getID()};

            m_sql.runSQLQuery(INSERT_BY_PROCEDURE ? "insertStatementProc" : "insertStatement", args);

        } catch (SQLException e1) {

            throw new RDFRDBException("Failed to register statement", e1);

        }

    }



    /**

     * List all the subject resources in the model

     * @param m the model whose subjects are to be listed

     * @return iterator over Resource objects

     */

    public ResultSetIterator listSubjects(ModelRDB m) throws RDFRDBException {

        try {

            Object[] args = null;

            if (supportsMultipleModels()) {

                args = new Object[] {m.getDBID().getID()};

            }

            ResultSetIterator it = new ResultSetResourceIterator(m, this);

            return m_sql.runSQLQuery("listSubjects", args, it);

        } catch (SQLException e1) {

            throw new RDFRDBException("Failure in listSubjects", e1);

        }

    }



    /**

     * List all namespaces referenced in the model

     * @return iterator over Strings

     */

    public ResultSetIterator listNamespaces(ModelRDB m) throws RDFRDBException {

        try {

            ResultSetStringIterator its = new ResultSetStringIterator();

            Object[] args = null;

            if (supportsMultipleModels()) {

                args = new Object[] {m.getDBID().getID()};

            }

            ResultSetIterator it = m_sql.runSQLQuery("listNamespaces", args, its);

            return it;

        } catch (SQLException e1) {

            throw new RDFRDBException("Failure in listNamespaces", e1);

        }

    }



    /**

     * List all the statements in the model which match the given subject,

     * predicate object - each of which can be null (meaning wildcard).

     */

    public ResultSetIterator listStatements(Resource subject, Property predicate, RDFNode object, ModelRDB m) throws RDFException {

        // Determine the SQL operation name

        String opname = "list";

        if (subject != null) opname += "S";

        if (predicate != null) opname += "P";

        if (object != null) opname += "O";

        try {

            Object[] args = statementArgs(subject, predicate, object, m);

            if (args == null) return new ResultSetIterator();

            return  m_sql.runSQLQuery(opname, args, new ResultSetStatementIterator(m, this));

        } catch (SQLException e) {

            throw new RDFRDBException("Internal sql failure in " + opname, e);

        }

    }



    /**

     * Check if a statement is in the database.

     */

    public boolean isStatementPresent(Resource subject, Property predicate, RDFNode object, ModelRDB m) throws RDFException {

        try {

            Object[] args = statementArgs(subject, predicate, object, m);

            if (args == null) return false;

            ResultSetIterator it = m_sql.runSQLQuery("checkStatement", args);

            boolean statementPresent = it.hasNext();

            it.close();

            return statementPresent;

        } catch (SQLException e) {

            throw new RDFRDBException("Internal sql failure in isStatementPresent", e);

        }

    }



    /**

     * List all the statements in the given moddel which make the given set of constraints

     * (a constraint set can be obtained using {@link #createConstraints}).

     */

    public ResultSetIterator listStatements(IConstraints constraints, ModelRDB m) throws RDFException {

        return constraints.runQuery(this, m);

    }



    /**

     * Create a constraint object appropriate for this type of database layout

     */

    public IConstraints createConstraints(ModelRDB model) {

        return new ConstraintsGeneric(model);

    }



    /**

     * Delete the statements in the model which match the given subject,

     * predicate object.

     */

    public void deleteStatement(Resource subject, Property predicate, RDFNode object, ModelRDB m) throws RDFException {

        try {

            Object[] args = statementArgs(subject, predicate, object, m);

            if (args == null) return;

            m_sql.runSQLQuery("deleteStatement", args);

        } catch (SQLException e) {

            throw new RDFRDBException("Internal sql failure in deleteStatement", e);

        }

    }



    /**

     * Fetch a resource, knowing its uri.

     * @todo insert cache manager here

     */

    public Resource getResource(String uri, ModelRDB m) throws RDFException {

        ResourceImplRDB r = new ResourceImplRDB(uri, m);

        //addResource(r);

        return r;

    }



    /**

     * Fetch a property, knowing its uri.

     * @todo insert cache manager here

     */

    public Property getProperty(String uri, ModelRDB m) throws RDFException {

        PropertyImplRDB r = new PropertyImplRDB(uri, m);

        //addResource(r);

        return r;

    }



//=======================================================================

// Multimodel support



    /**

     * Returns true of the database layout supports multiple RDF models

     * in the same database.

     */

    public boolean supportsMultipleModels() {

        return false;

    }



    /**

     * Allocate a new databaseID for a new model in the same database.

     */

    public IDBID allocateModelID(String modelName) throws RDFRDBException {

        return new DBIDInt(0);

    }



    /**

     * Return the databaseID for an existing named model within the database.

     * Throws RDFRDBException if the driver does not support multiple models per db.

     */

    public IDBID getModelID(String modelName) throws RDFRDBException {

        throw new RDFRDBException("Generic driver does not support multiple models in one database.\nCaught attempting to access model: " + modelName);

    }



    /**

     * List the names of all models in this database

     */

    public Iterator listModels() throws RDFRDBException {

        return (new ArrayList()).iterator();        // Empty iterator since does support multiple models

    }



//=======================================================================

// Support for registering/looking up resources, literals, namespaces

// For most categories we need operations to:

//    getXID - determine the DBID, if available

//    allocateXID - allocate a new DBID, if possible

//    getX   - reconstruct the object from its DBID

//    addX   - add to the database, finds DBID as side effect

//=======================================================================



    /**

     * General ID allocate stub.

     * Calls the given SQL operation to perform the allocation.

     */

    protected IDBID allocateID(String opname) throws RDFRDBException {

        try {

            ResultSetIterator it = m_sql.runSQLQuery(opname, new Object[] {});

            if (it.hasNext()) {

                return wrapDBID(it.getSingleton());

            } else {

                throw new RDFRDBException("Failed to allocate ID");

            }

        } catch (SQLException e) {

            throw new RDFRDBException("Internal sql error", e);

        }

    }



    /**

     * Return the namespace string corresponding to a given namespace id.

     * Can be null if the ID can from a hash function and namespace isn't registered yet.

     */

    public String getNamespace(IDBID nsid) throws RDFRDBException {

        String namespace = (String) m_namespaceMap.get(nsid);

        if (namespace == null) {

            try {

                // retrieve from database

                ResultSetIterator rs = m_sql.runSQLQuery("getNamespace", new Object[] {nsid.getID()});

                if (rs.hasNext()) {

                    namespace = (String)rs.getSingleton();

                } else {

                    namespace = null;

                    rs.close();

                }

                if (namespace != null) cacheNamespaceId(namespace, nsid);

            } catch (SQLException e) {

                throw new RDFRDBException("Internal error fetching namespace name", e);

            }

        }

        return namespace;

    }



    /**

     * Return the namespace ID for the given namespace or null if the namespace is not yet registered.

     */

    protected IDBID getNamespaceID(String namespace) throws RDFRDBException{

        if (m_namespaceIDMap.containsKey(namespace)) {

            return (IDBID) m_namespaceIDMap.get(namespace);

        } else {

            try {

                ResultSetIterator it = m_sql.runSQLQuery("getNamespaceID", new Object[] {namespace});

                if (it.hasNext()) {

                    IDBID id = wrapDBID(it.getSingleton());

                    cacheNamespaceId(namespace, id);

                    return id;

                } else {

                    return null;

                }

            } catch (SQLException e) {

                throw new RDFRDBException("Internal sql error", e);

            }

        }

    }



    /**

     * Return the namespace ID, if it doesn't have one yet then allocate one.

     */

    protected IDBID allocateNamespaceID(String namespace) throws RDFRDBException{

        if (m_namespaceIDMap.containsKey(namespace)) {

            return (IDBID) m_namespaceIDMap.get(namespace);

        } else {

            return allocateID("allocateNamespaceID");

        }

    }





    /**

     * Register a namespace with the database and return its ID.

     * Won't add duplicates.

     */

    protected IDBID addNamespace(String namespace) throws RDFRDBException {

        if (m_namespaceIDMap.containsKey(namespace)) {

            return (IDBID) m_namespaceIDMap.get(namespace);

        } else {

            Object[] args = null;

            IDBID id = null;

            if (!SKIP_DUPLICATE_CHECK) {

                id = getNamespaceID(namespace);

                if (id != null) return id;

            }

            if (!SKIP_ALLOCATE_ID) {

                id = allocateNamespaceID(namespace);

                args = new Object[] {id.getID(), namespace};

            } else {

                args = new Object[] {namespace};

            }

            try {

                if (INSERT_BY_PROCEDURE) {

                    ResultSetIterator it = m_sql.runSQLQuery("insertNamespaceProc", args);

                    if (it != null && it.hasNext())

                        id = wrapDBID(it.getSingleton());

                } else {

                    m_sql.runSQLQuery("insertNamespace", args);

                }

                if (id == null) {

                    id = getNamespaceID(namespace);

                }

                cacheNamespaceId(namespace, id);

                return id;

            } catch (SQLException e) {

                throw new RDFRDBException("Internal sql error", e);

            }

        }

    }



    /**

     * Fetch a literal just knowing its literal rdb-id.

     * Can be null if the ID can from a hash function and literal isn't registered yet.

     */

    public Literal getLiteral(IDBID id) throws RDFException {

        try {

            PreparedStatement ps = m_sql.getPreparedSQLStatement("getLiteral");

            ps.setObject(1, id.getID());

            ResultSet rs = ps.executeQuery();

            if (!rs.next()) {

                m_sql.returnPreparedSQLStatement(ps, "getLiteral");

                return null;

            }

            Object blob = rs.getObject(1);

            String literal = null;

            if (blob == null) {

                // No blob so the literal must have fitted within the index field

                literal = rs.getString(2);

            } else {

                InputStream blobin = null;

                if (blobin instanceof InputStream) {

                    blobin = (InputStream) blob;

                } else {

                    // Probably postgresql with it's BLOB=OID stuff and a broken jdbc driver

                    // This re-open of an arg is not guaranteed to work for all drivers but

                    // this code is probably only reached for the broken postgresql case which

                    // does support arg reopening.

                    blobin = rs.getBinaryStream(1);

                }

                int len = blobin.read() & 0xff;

                len |= ((blobin.read() & 0xff) << 8);

                len |= ((blobin.read() & 0xff) << 16);

                len |= ((blobin.read() & 0xff) << 24);

                //System.out.println("Len = " + len);

                byte[] data = new byte[len];

                int read = 0;

                while (read < len) {

                    int got = blobin.read(data, read, len-read);

                    if (got == -1) {

                        throw new RDFRDBException("Premature end of blob in large literal, got " + read);

                    }

                    read += got;

                }

                blobin.close();

                literal = new String(data, "UTF-8");

            }

            String language = rs.getString(3);

            int wellFormedW = rs.getInt(4);

            rs.close();

            m_sql.returnPreparedSQLStatement(ps, "getLiteral");

            return new LiteralImpl(literal, language, wellFormedW == 1);

        } catch (Exception e) {

            throw new RDFRDBException("Internal sql error", e);

        }

    }



    /**

     * Register a literal

     * @return the db index of the added literal

     */

    protected IDBID addLiteral(Literal l) throws RDFRDBException {

        IDBID id = null;

        if (!SKIP_DUPLICATE_CHECK) {

            id = getLiteralID(l);

            if (id != null) return id;

        }

        try {

            int val = 0;               // Integer translation of l, if possible

            boolean isInt = false; // true if literal can be interpreted as an integer

            // Check if the literal can be translated to an int

            try {

                val = Integer.parseInt(l.toString());

                isInt = true;

            } catch (NumberFormatException e) {

                isInt = false;

            }

            String opname = isInt ? "insertLiteralInt" : "insertLiteral";

            if (INSERT_BY_PROCEDURE) opname = opname + "Proc";

            PreparedStatement ps = m_sql.getPreparedSQLStatement(opname);

            int argi = 1;

            if (!SKIP_ALLOCATE_ID) {

                id = allocateLiteralID(l);

                ps.setObject(argi++, id.getID());

            }

            String lit = l.toString();

            int len = lit.length();

            if (len >= MAX_LITERAL) {

                // Literal too long to fit, insert a subset for indexing and put whole into a blob

                ps.setString(argi++, getLiteralIdx(lit));

                // First convert the literal to a UTF-16 encoded byte array

                // (this wouldn't be needed for jdbc 2.0 drivers but not all db's have them)

                byte[] temp = lit.getBytes("UTF-8");

                int lenb = temp.length;

                //System.out.println("utf-16 len = " + lenb);

                byte[] litData = new byte[lenb + 4];

                litData[0] = (byte)(lenb & 0xff);

                litData[1] = (byte)((lenb >> 8) & 0xff);

                litData[2] = (byte)((lenb >> 16) & 0xff);

                litData[3] = (byte)((lenb >> 24) & 0xff);

                System.arraycopy(temp, 0, litData, 4, lenb);

                ps.setBinaryStream(argi++, new ByteArrayInputStream(litData), litData.length);

            } else {

                ps.setString(argi++, lit);

                ps.setNull(argi++, Types.BLOB);

            }

            ps.setString(argi++, l.getLanguage());

            ps.setInt(argi++, l.getWellFormed() ? 1 : 0);

            if (isInt) {

                ps.setInt(argi++, val);

            }

            if (INSERT_BY_PROCEDURE) {

                ResultSet rs = ps.executeQuery();

                ResultSetIterator it = new ResultSetIterator(rs, ps, m_sql, opname);

                if (it != null && it.hasNext())

                    id = wrapDBID(it.getSingleton());

            } else {

                ps.executeUpdate();

                m_sql.returnPreparedSQLStatement(ps, opname);

            }

            if (id == null)

                id = getLiteralID(l);

            return id;

        } catch (Exception e1) {

            // /* DEBUG */ System.out.println("Problem on literal (l=" + l.toString().length() + "): " + l);

            /* DEBUG */ System.out.println("Problem on literal (l=" + l.toString().length() + ") ");

            /* DEBUG */ System.out.println("ID is: " + id);

            throw new RDFRDBException("Failed to register literal", e1);

        }

    }



    /**

     * Return the database ID for the literal, if it exists

     */

    public IDBID getLiteralID(Literal l) throws RDFRDBException {

        try {

            String ls = l.toString();

            if (ls.length() >= MAX_LITERAL) {

                ls = getLiteralIdx(ls);

            }

            ResultSetIterator rs = m_sql.runSQLQuery("getLiteralID", new Object[] {ls, l.getLanguage()});

            if (rs.hasNext()) {

                return wrapDBID(rs.getSingleton());

            } else {

                return null;

            }

        } catch (SQLException e1) {

            // /* DEBUG */ System.out.println("Literal truncation (" + l.toString().length() + ") " + l.toString().substring(0, 150));

            throw new RDFRDBException("Failed to find literal", e1);

        }

    }



    /**

     * Return the database ID for the literal and allocate one of necessary

     */

    protected IDBID allocateLiteralID(Literal l) throws RDFRDBException {

        return allocateID("allocateLiteralID");

    }



    /**

     * Register a Resource URI in the database and record the unique ID

     * for it.

     */

    protected IDBID addResource(Resource resource) throws RDFException {

        boolean rdbOK = (resource instanceof ResourceImplRDB);

        ResourceImplRDB resourceRDB = rdbOK ? (ResourceImplRDB) resource : null;

        IDBID id = rdbOK ? resourceRDB.getRDBId() : null;

        if (id != null) return id;

        Object[] args = null;

        int argi = 0;

        if (!SKIP_DUPLICATE_CHECK) {

            id = getResourceID(resource);

            if (id != null) return id;

        }

        if (!SKIP_ALLOCATE_ID) {

            id = allocateResourceID(resource);

            args = new Object[3];

            args[argi++] = id.getID();

        } else {

            args = new Object[2];

        }

        args[argi++] = resource.isAnon() ? resource.getId().toString() : resource.getLocalName();

        IDBID nsid = rdbOK ? resourceRDB.getRDBNSId(): null;

        if (nsid == null) {

            nsid = resource.isAnon() ? m_anonNSId : addNamespace(resource.getNameSpace());

            if (rdbOK) resourceRDB.setRDBNSId(nsid);

        }

        args[argi++] = nsid.getID();

        try {

            if (INSERT_BY_PROCEDURE) {

                ResultSetIterator rs = m_sql.runSQLQuery("insertResourceProc", args);

                if (rs != null && rs.hasNext())

                    id = wrapDBID(rs.getSingleton());

            } else {

                m_sql.runSQLQuery("insertResource", args);

            }

        } catch (SQLException e1) {

            // /* DEBUG */ System.out.println("Resource trucation: " + resource);

            throw new RDFRDBException("Failed to register resource", e1);

        }

        if (id == null) {

            id = getResourceID(resource);

        }

        if (rdbOK) resourceRDB.setRDBId(id);

        return id;

    }



    /**

     * Lookup the unique ID for a resource.

     * Return null if it does not have one yet.

     */

    public IDBID getResourceID(Resource resource) throws RDFException {

        try {

            ResourceImplRDB resourceRDB = (ResourceImplRDB) resource;

            IDBID id = resourceRDB.getRDBId();

            if (id != null) return id;

            String namespace = resource.isAnon() ? ANON_NAMESPACE : resource.getNameSpace();

            IDBID nsid = getNamespaceID(namespace);

            if (nsid == null) return null;

            resourceRDB.setRDBNSId(nsid);

            String localname = resource.isAnon() ? resource.getId().toString() : resource.getLocalName();

            ResultSetIterator rs = m_sql.runSQLQuery("getResourceID", new Object[] {localname, nsid.getID()});

            if (rs.hasNext()) {

                id = wrapDBID(rs.getSingleton());

                resourceRDB.setRDBId(id);

                return id;

            } else {

                return null;

            }

        } catch (SQLException e1) {

            // /* DEBUG */ System.out.println("Literal truncation (" + l.toString().length() + ") " + l.toString().substring(0, 150));

            throw new RDFRDBException("Failed to find resource", e1);

        }

    }



    /**

     * Allocate a new ID for the resource.

     */

    protected IDBID allocateResourceID(Resource resource) throws RDFException {

        return allocateID("allocateResourceID");

    }



    /**

     * Fetch a resource just knowing its resource rdb-id.

     * Can be null if the ID can from a hash function and resource isn't registered yet.

     * @todo insert cache manager here

     */

    public Resource getResource(IDBID id, ModelRDB m) throws RDFException {

        try {

            ResultSetIterator it = m_sql.runSQLQuery("getResource", new Object[] {id.getID()});

            if (it.hasNext()) {

                List row = (List) it.next();

                String localname = (String) row.get(0);

                IDBID nsid = wrapDBID(row.get(1));

                it.close();

                return makeResource(id, localname, nsid, m);

            } else {

                return null;

            }

        } catch (SQLException e) {

            throw new RDFRDBException("Internal sql error", e);

        }

    }



    /**

     * Fetch a property just knowing its resource rdb-id.

     * Can be null if the ID can from a hash function and resource isn't registered yet.

     * @todo insert cache manager here

     */

    public Property getProperty(IDBID id, ModelRDB m) throws RDFException {

        try {

            ResultSetIterator it = m_sql.runSQLQuery("getResource", new Object[] {id.getID()});

            if (it.hasNext()) {

                List row = (List) it.next();

                String localname = (String) row.get(0);

                IDBID nsid = wrapDBID(row.get(1));

                it.close();

                PropertyImplRDB p = new PropertyImplRDB(getNamespace(nsid), localname, m);

                p.setRDBId(id);

                p.setRDBNSId(nsid);

                return p;

            } else {

                return null;

            }

        } catch (SQLException e) {

            throw new RDFRDBException("Internal sql error", e);

        }

    }



//=======================================================================

// Supporting machinery



    /**

     * Extract an indexable sub-string from a literal.

     * This is used for literals that are too long to index for the given

     * database

     */

    protected String getLiteralIdx(String literal) {

        String literal_idx = literal;

        if (literal.length() > (MAX_LITERAL - 16)) {

            // Some databases can't index long literals so literals are stored in two parts,

            // a shorter indexable string plus the full string

            CRC32 checksum = new CRC32();

            checksum.update(literal.getBytes());

            literal_idx = literal.substring(0, MAX_LITERAL-16) + Long.toHexString(checksum.getValue());

        }

        return literal_idx;

    }



    /**

     * Convert the raw SQL object used to store a database identifier into a java object

     * which meets the IDBID interface

     */

    public IDBID wrapDBID(Object id) throws RDFRDBException {

        if (id instanceof Number) {

            return new DBIDInt(((Number)id).intValue());

        } else {

            /** @todo modify signature so this can raise an exception, easy once we switch to non-checked exceptions */

            throw new RDFRDBException("Unexpected DB identifier type: " + id);

            //return null;

        }

    }



    /**

     * Wrap up a boolean flag as a object which the jdbc driver can assert into a boolean/short column

     */

    public Object wrapFlag(boolean flag) {

        return  flag ? new Short((short)1) : new Short((short)0);

    }



    /**

     * Cache the given namespace - id mapping

     */

    protected void cacheNamespaceId(String namespace, IDBID id) {

        if (!m_namespaceMap.containsKey(id)) {

            m_namespaceMap.put(id, namespace);

            m_namespaceIDMap.put(namespace, id);

        }

    }



    /**

     * Re-create a resource from its local name and namespace identifier

     * @todo add resource cache in here

     */

    public Resource makeResource(IDBID id, String localname, IDBID nsid, ModelRDB m) throws RDFException {

        ResourceImplRDB res = null;

        if (nsid.equals(m_anonNSId)) {

            res = new ResourceImplRDB(new AnonId(localname), m);

        } else {

            res = new ResourceImplRDB(getNamespace(nsid)+localname, m);

        }

        res.setRDBId(id);

        res.setRDBNSId(nsid);

        return res;

    }



    /**

     * Force a load of the jdbc driver. This is equivalent to

     * called <code>Class.forname(...)</code> on the JDBC driver

     * classname except that the classname is determined from

     * the jena RDB configuration file.

     * @throws RDFRDBException if the driver fails to load

     */

    public void loadDatabaseDriver() throws RDFRDBException {

        try {

            Class.forName (DRIVER_NAME);

        } catch (ClassNotFoundException e) {

            throw new RDFRDBException ("Unable to load driver: " + DRIVER_NAME);

        }

    }



    /**

     * Package the arguments to a statement function like deleteStatement or isStatementPresent.

     * Returns null if any of the components is not present in the db.

     */

    protected Object[] statementArgs(Resource subject, Property predicate, RDFNode object, ModelRDB m) throws RDFException {

        int argi = 0;

        int argcount = 0;

        if (subject != null) argcount++;

        if (predicate != null) argcount++;

        if (object != null) argcount += 2;

        if (supportsMultipleModels()) argcount++;

        Object[] args = new Object[argcount];

        if (subject != null) {

            IDBID sid = getResourceID(subject);

            if (sid == null) return null;

            args[argi++] = sid.getID();

        }

        if (predicate != null) {

            IDBID pid = getResourceID(predicate);

            if (pid == null) return null;

            args[argi++] = pid.getID();

        }

        if (object != null) {

            if (object instanceof Literal) {

                IDBID lid = getLiteralID((Literal)object);

                if (lid == null) {

                    return null;

                }

                args[argi++] = lid.getID();

                args[argi++] = wrapFlag(true);

            } else {

                IDBID lid = getResourceID((Resource)object);

                if (lid == null) return null;

                args[argi++] = lid.getID();

                args[argi++] = wrapFlag(false);

            }

        }

        if (supportsMultipleModels()) {

            args[argi++] = m.getDBID().getID();

        }

        return args;

    }





    /**

     * Hint to the store that the given property could be stored as an

     * attribute table.

     */

    public void hintPropertyTable(Property p) throws RDFException {

        // Default implementation is to ignore the hint

    }

}



/*

 *  (c) Copyright Hewlett-Packard Company 2000, 2001

 *  All rights reserved.

 *

 * Redistribution and use in source and binary forms, with or without

 * modification, are permitted provided that the following conditions

 * are met:

 * 1. Redistributions of source code must retain the above copyright

 *    notice, this list of conditions and the following disclaimer.

 * 2. Redistributions in binary form must reproduce the above copyright

 *    notice, this list of conditions and the following disclaimer in the

 *    documentation and/or other materials provided with the distribution.

 * 3. The name of the author may not be used to endorse or promote products

 *    derived from this software without specific prior written permission.



 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR

 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES

 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.

 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,

 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT

 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,

 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY

 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT

 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF

 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */

