/* -*-Java-*-

*******************************************************************

*

* File:         DriverOracleMMGeneric.java

* RCS:          $Header: /cvsroot/jena/jena/src/com/hp/hpl/mesa/rdf/jena/rdb/DriverOracleMMGeneric.java,v 1.1 2002/03/01 14:26:39 der Exp $

* Author:       Dave Reynolds, HPLabs, Bristol

* Created:      20/2/02

* Modified:     $Date: 2002/03/01 14:26:39 $

* Status:       Experimental (Do Not Distribute)

*

* (c) Copyright 2001, Hewlett-Packard Company, all rights reserved.

* [See end of file]

*********************************************************************/



//=======================================================================

// Package

package com.hp.hpl.mesa.rdf.jena.rdb;



//=======================================================================

// Imports

import com.hp.hpl.mesa.rdf.jena.common.*;

import com.hp.hpl.mesa.rdf.jena.model.*;

import java.io.*;

import java.sql.*;

import java.util.*;

import com.hp.hpl.jena.util.Log;



//=======================================================================

/**

* Customize the MMGeneric driver for use with Oracle.

* The key issues with Oracle are:

* (1) Possible return of bignums for IDs - now handled in generic driver

* (2) Empty strings are stored as nulls and so do not match other empty strings,

*     this breaks all the "langauge" field processing in literals in several ways.

*

* @author <a href="mailto:der@hplb.hpl.hp.com">Dave Reynolds</a>

* @version $Revision: 1.1 $ on $Date: 2002/03/01 14:26:39 $

*/



public class DriverOracleMMGeneric extends DriverGenericMMGeneric implements IRDBDriver {



    /** table of default sql statements */

    protected static Properties m_defaultMMSQL = null;



    /**

     * Create a bare instance of the driver. It is not functional until a

     * database connection has been supplied via setConnection.

     */

    public DriverOracleMMGeneric() {

        super();

        // Set configuration parameters

        DRIVER_NAME = "oracle.jdbc.driver.OracleDriver";

        LAYOUT_TYPE = "MMGeneric";

        DATABASE_TYPE = "Oracle";

        SQL_FILE = "etc/oracle_mmgeneric.sql";

        MAX_LITERAL = 1000;

        ID_SQL_TYPE = "INTEGER";

    }



//=======================================================================

// Patched functions to adapt to oracle jdbc driver expectations



    /**

     * Return the database ID for the literal, if it exists

     */

    public IDBID getLiteralID(Literal l) throws RDFRDBException {

        try {

            String ls = l.toString();

            if (ls.length() >= MAX_LITERAL) {

                ls = getLiteralIdx(ls);

            }

            String lang = l.getLanguage();

            boolean nullLang = (lang == null) || (lang.equals(""));

            // Oracle (thin driver at least) fails to match empty strings so have to watch out for when lang is empty

            String opName = nullLang ? "getLiteralIDNoLang" : "getLiteralID";

            PreparedStatement ps = m_sql.getPreparedSQLStatement(opName);

            ps.setString(1, ls);

            if (!nullLang) ps.setString(2, lang);

            ResultSet rs = ps.executeQuery();

            IDBID result = null;

            if (rs.next()) {

                result = wrapDBID(rs.getObject(1));

            }

            rs.close();

            m_sql.returnPreparedSQLStatement(ps, opName);

            return result;

        } catch (SQLException e1) {

            // /* DEBUG */ System.out.println("Literal truncation (" + l.toString().length() + ") " + l.toString().substring(0, 150));

            throw new RDFRDBException("Failed to find literal", e1);

        }

    }



    /**

     * Wrap up a boolean flag as a object which the jdbc driver can assert into a boolean/short column

     */

    public Object wrapFlag(boolean flag) {

        return  flag ? new Integer(1) : new Integer(0);

    }



    /**

     * Fetch a literal just knowing its literal rdb-id.

     * Can be null if the ID can from a hash function and literal isn't registered yet.

     * Patches up any mangled lang fields from Oracle.

     */

    public Literal getLiteral(IDBID id) throws RDFException {

        Literal l = super.getLiteral(id);

        if (l != null && l.getLanguage() == null) {

            // At this point Oracle has retrieved something which was an empty string as a null

            // Can't just fixup inplace (no setLanguage operator on LiteralImpl so create new one.

            // The overhead is probably trivial compared to the DB call anyway.

            return new LiteralImpl(l.getString(), "", l.getWellFormed());

        } else {

            return l;

        }

    }



    /**

     * Lookup the unique ID for a resource.

     * Return null if it does not have one yet.

     */

    public IDBID getResourceID(Resource resource) throws RDFException {

        try {

            ResourceImplRDB resourceRDB = (ResourceImplRDB) resource;

            IDBID id = resourceRDB.getRDBId();

            if (id != null) return id;

            String namespace = resource.isAnon() ? ANON_NAMESPACE : resource.getNameSpace();

            IDBID nsid = getNamespaceID(namespace);

            if (nsid == null) return null;

            resourceRDB.setRDBNSId(nsid);

            String localname = resource.isAnon() ? resource.getId().toString() : resource.getLocalName();

            boolean nullName = localname == null || localname.equals("");

            String opName = nullName ? "getResourceIDNoName" : "getResourceID";

            Object[] args = nullName ? new Object[] {nsid.getID()} : new Object[] {localname, nsid.getID()};

            ResultSetIterator rs = m_sql.runSQLQuery(opName, args);

            if (rs.hasNext()) {

                id = wrapDBID(rs.getSingleton());

                resourceRDB.setRDBId(id);

                return id;

            } else {

                return null;

            }

        } catch (SQLException e1) {

            // /* DEBUG */ System.out.println("Literal truncation (" + l.toString().length() + ") " + l.toString().substring(0, 150));

            throw new RDFRDBException("Failed to find resource", e1);

        }

    }



}



/*

 *  (c) Copyright Hewlett-Packard Company 2000, 2001

 *  All rights reserved.

 *

 * Redistribution and use in source and binary forms, with or without

 * modification, are permitted provided that the following conditions

 * are met:

 * 1. Redistributions of source code must retain the above copyright

 *    notice, this list of conditions and the following disclaimer.

 * 2. Redistributions in binary form must reproduce the above copyright

 *    notice, this list of conditions and the following disclaimer in the

 *    documentation and/or other materials provided with the distribution.

 * 3. The name of the author may not be used to endorse or promote products

 *    derived from this software without specific prior written permission.



 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR

 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES

 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.

 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,

 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT

 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,

 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY

 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT

 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF

 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

 */

